/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.akregator.part.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "partadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class PartAdaptor
 */

PartAdaptor::PartAdaptor(Akregator::Part *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

PartAdaptor::~PartAdaptor()
{
    // destructor
}

void PartAdaptor::addFeed()
{
    // handle method call org.kde.akregator.part.addFeed
    parent()->addFeed();
}

void PartAdaptor::addFeedsToGroup(const QStringList &lst, const QString &feedname)
{
    // handle method call org.kde.akregator.part.addFeedsToGroup
    parent()->addFeedsToGroup(lst, feedname);
}

void PartAdaptor::exportFile(const QString &url)
{
    // handle method call org.kde.akregator.part.exportFile
    parent()->exportFile(url);
}

void PartAdaptor::fetchAllFeeds()
{
    // handle method call org.kde.akregator.part.fetchAllFeeds
    parent()->fetchAllFeeds();
}

void PartAdaptor::fetchFeedUrl(const QString &url)
{
    // handle method call org.kde.akregator.part.fetchFeedUrl
    parent()->fetchFeedUrl(url);
}

bool PartAdaptor::handleCommandLine(const QStringList &args)
{
    // handle method call org.kde.akregator.part.handleCommandLine
    return parent()->handleCommandLine(args);
}

void PartAdaptor::openStandardFeedList()
{
    // handle method call org.kde.akregator.part.openStandardFeedList
    parent()->openStandardFeedList();
}

void PartAdaptor::saveSettings()
{
    // handle method call org.kde.akregator.part.saveSettings
    parent()->saveSettings();
}


#include "moc_partadaptor.cpp"
