/****************************************************************************
** Meta object code from reading C++ file 'akregator_part.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/akregator_part.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'akregator_part.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator4PartE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::Part::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator4PartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::Part",
        "signalSettingsChanged",
        "",
        "signalArticlesSelected",
        "QList<Akregator::Article>",
        "saveSettings",
        "slotSaveFeedList",
        "fileImport",
        "fileExport",
        "showOptions",
        "showNotificationOptions",
        "slotAutoSave",
        "slotOnShutdown",
        "slotSettingsChanged",
        "slotSetStatusText",
        "statusText",
        "feedListLoaded",
        "QSharedPointer<Akregator::FeedList>",
        "list",
        "flushAddFeedRequests",
        "slotRestoreSession",
        "Akregator::CrashWidget::CrashAction",
        "type"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalSettingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalArticlesSelected'
        QtMocHelpers::SignalData<void(const QList<Akregator::Article> &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveFeedList'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileImport'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileExport'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showOptions'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showNotificationOptions'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAutoSave'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnShutdown'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetStatusText'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'feedListLoaded'
        QtMocHelpers::SlotData<void(const QSharedPointer<Akregator::FeedList> &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'flushAddFeedRequests'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRestoreSession'
        QtMocHelpers::SlotData<void(Akregator::CrashWidget::CrashAction)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Part, qt_meta_tag_ZN9Akregator4PartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::Part::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::Part::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4PartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4PartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator4PartE_t>.metaTypes,
    nullptr
} };

void Akregator::Part::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Part *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalSettingsChanged(); break;
        case 1: _t->signalArticlesSelected((*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[1]))); break;
        case 2: _t->saveSettings(); break;
        case 3: _t->slotSaveFeedList(); break;
        case 4: _t->fileImport(); break;
        case 5: _t->fileExport(); break;
        case 6: _t->showOptions(); break;
        case 7: _t->showNotificationOptions(); break;
        case 8: _t->slotAutoSave(); break;
        case 9: _t->slotOnShutdown(); break;
        case 10: _t->slotSettingsChanged(); break;
        case 11: _t->slotSetStatusText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->feedListLoaded((*reinterpret_cast<std::add_pointer_t<QSharedPointer<Akregator::FeedList>>>(_a[1]))); break;
        case 13: _t->flushAddFeedRequests(); break;
        case 14: _t->slotRestoreSession((*reinterpret_cast<std::add_pointer_t<Akregator::CrashWidget::CrashAction>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Part::*)()>(_a, &Part::signalSettingsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QList<Akregator::Article> & )>(_a, &Part::signalArticlesSelected, 1))
            return;
    }
}

const QMetaObject *Akregator::Part::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::Part::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4PartE_t>.strings))
        return static_cast<void*>(this);
    return KParts::Part::qt_metacast(_clname);
}

int Akregator::Part::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::Part::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Akregator::Part::signalSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Akregator::Part::signalArticlesSelected(const QList<Akregator::Article> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
