/********************************************************************************
** Form generated from reading UI file 'settings_archive.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_ARCHIVE_H
#define UI_SETTINGS_ARCHIVE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace Akregator {

class Ui_SettingsArchive
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QRadioButton *rb_DisableArchiving;
    QRadioButton *rb_KeepAllArticles;
    QHBoxLayout *hboxLayout;
    QRadioButton *rb_LimitArticleNumber;
    QSpinBox *kcfg_MaxArticleNumber;
    QHBoxLayout *hboxLayout1;
    QRadioButton *rb_LimitArticleAge;
    QSpinBox *kcfg_MaxArticleAge;
    QSpacerItem *horizontalSpacer;
    QCheckBox *kcfg_DoNotExpireImportantArticles;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *Akregator__SettingsArchive)
    {
        if (Akregator__SettingsArchive->objectName().isEmpty())
            Akregator__SettingsArchive->setObjectName("Akregator__SettingsArchive");
        Akregator__SettingsArchive->resize(355, 219);
        verticalLayout = new QVBoxLayout(Akregator__SettingsArchive);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(Akregator__SettingsArchive);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        rb_DisableArchiving = new QRadioButton(groupBox);
        rb_DisableArchiving->setObjectName("rb_DisableArchiving");

        gridLayout->addWidget(rb_DisableArchiving, 3, 0, 1, 1);

        rb_KeepAllArticles = new QRadioButton(groupBox);
        rb_KeepAllArticles->setObjectName("rb_KeepAllArticles");

        gridLayout->addWidget(rb_KeepAllArticles, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        rb_LimitArticleNumber = new QRadioButton(groupBox);
        rb_LimitArticleNumber->setObjectName("rb_LimitArticleNumber");

        hboxLayout->addWidget(rb_LimitArticleNumber);

        kcfg_MaxArticleNumber = new QSpinBox(groupBox);
        kcfg_MaxArticleNumber->setObjectName("kcfg_MaxArticleNumber");
        kcfg_MaxArticleNumber->setEnabled(false);
        kcfg_MaxArticleNumber->setMaximum(999999999);
        kcfg_MaxArticleNumber->setValue(200);

        hboxLayout->addWidget(kcfg_MaxArticleNumber);


        gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        rb_LimitArticleAge = new QRadioButton(groupBox);
        rb_LimitArticleAge->setObjectName("rb_LimitArticleAge");

        hboxLayout1->addWidget(rb_LimitArticleAge);

        kcfg_MaxArticleAge = new QSpinBox(groupBox);
        kcfg_MaxArticleAge->setObjectName("kcfg_MaxArticleAge");
        kcfg_MaxArticleAge->setEnabled(false);
        kcfg_MaxArticleAge->setMaximum(999999999);
        kcfg_MaxArticleAge->setValue(30);

        hboxLayout1->addWidget(kcfg_MaxArticleAge);


        gridLayout->addLayout(hboxLayout1, 2, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 1, 1, 1);


        verticalLayout->addWidget(groupBox);

        kcfg_DoNotExpireImportantArticles = new QCheckBox(Akregator__SettingsArchive);
        kcfg_DoNotExpireImportantArticles->setObjectName("kcfg_DoNotExpireImportantArticles");

        verticalLayout->addWidget(kcfg_DoNotExpireImportantArticles);

        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);

        verticalLayout->setStretch(2, 1);

        retranslateUi(Akregator__SettingsArchive);

        QMetaObject::connectSlotsByName(Akregator__SettingsArchive);
    } // setupUi

    void retranslateUi(QWidget *Akregator__SettingsArchive)
    {
        groupBox->setTitle(tr2i18n("Default Archive Settings", nullptr));
        rb_DisableArchiving->setText(tr2i18n("Disa&ble archiving", nullptr));
        rb_KeepAllArticles->setText(tr2i18n("&Keep all articles", nullptr));
        rb_LimitArticleNumber->setText(tr2i18n("Limit feed archi&ve size to:", nullptr));
        rb_LimitArticleAge->setText(tr2i18n("Delete articles older than: ", nullptr));
        kcfg_DoNotExpireImportantArticles->setText(tr2i18n("Do not expire important articles", nullptr));
        (void)Akregator__SettingsArchive;
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class SettingsArchive: public Ui_SettingsArchive {};
} // namespace Ui
} // namespace Akregator

#endif // SETTINGS_ARCHIVE_H

