/********************************************************************************
** Form generated from reading UI file 'settings_general.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_GENERAL_H
#define UI_SETTINGS_GENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace Akregator {

class Ui_SettingsGeneral
{
public:
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *kcfg_ShowTrayIcon;
    QCheckBox *kcfg_ShowUnreadInTaskbar;
    QCheckBox *kcfg_HideReadFeeds;
    QCheckBox *kcfg_LockFeedsInPlace;
    QCheckBox *kcfg_AutoExpandFolders;
    QCheckBox *kcfg_UseNotifications;
    QCheckBox *kcfg_DisableSaveAsNotification;
    QCheckBox *kcfg_UseIntervalFetch;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *spacerItem;
    QLabel *autoFetchIntervalLabel;
    QSpinBox *kcfg_AutoFetchInterval;
    QSpacerItem *horizontalSpacer;
    QCheckBox *kcfg_PlasmaActivitySupport;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_MarkAllFeedsReadOnStartup;
    QCheckBox *kcfg_FetchOnStartup;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_UseHTMLCache;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Akregator__SettingsGeneral)
    {
        if (Akregator__SettingsGeneral->objectName().isEmpty())
            Akregator__SettingsGeneral->setObjectName("Akregator__SettingsGeneral");
        Akregator__SettingsGeneral->resize(362, 474);
        verticalLayout_4 = new QVBoxLayout(Akregator__SettingsGeneral);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        groupBox_3 = new QGroupBox(Akregator__SettingsGeneral);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_3 = new QVBoxLayout(groupBox_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        kcfg_ShowTrayIcon = new QCheckBox(groupBox_3);
        kcfg_ShowTrayIcon->setObjectName("kcfg_ShowTrayIcon");

        verticalLayout_3->addWidget(kcfg_ShowTrayIcon);

        kcfg_ShowUnreadInTaskbar = new QCheckBox(groupBox_3);
        kcfg_ShowUnreadInTaskbar->setObjectName("kcfg_ShowUnreadInTaskbar");

        verticalLayout_3->addWidget(kcfg_ShowUnreadInTaskbar);

        kcfg_HideReadFeeds = new QCheckBox(groupBox_3);
        kcfg_HideReadFeeds->setObjectName("kcfg_HideReadFeeds");

        verticalLayout_3->addWidget(kcfg_HideReadFeeds);

        kcfg_LockFeedsInPlace = new QCheckBox(groupBox_3);
        kcfg_LockFeedsInPlace->setObjectName("kcfg_LockFeedsInPlace");

        verticalLayout_3->addWidget(kcfg_LockFeedsInPlace);

        kcfg_AutoExpandFolders = new QCheckBox(groupBox_3);
        kcfg_AutoExpandFolders->setObjectName("kcfg_AutoExpandFolders");

        verticalLayout_3->addWidget(kcfg_AutoExpandFolders);

        kcfg_UseNotifications = new QCheckBox(groupBox_3);
        kcfg_UseNotifications->setObjectName("kcfg_UseNotifications");

        verticalLayout_3->addWidget(kcfg_UseNotifications);

        kcfg_DisableSaveAsNotification = new QCheckBox(groupBox_3);
        kcfg_DisableSaveAsNotification->setObjectName("kcfg_DisableSaveAsNotification");

        verticalLayout_3->addWidget(kcfg_DisableSaveAsNotification);

        kcfg_UseIntervalFetch = new QCheckBox(groupBox_3);
        kcfg_UseIntervalFetch->setObjectName("kcfg_UseIntervalFetch");

        verticalLayout_3->addWidget(kcfg_UseIntervalFetch);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem);

        autoFetchIntervalLabel = new QLabel(groupBox_3);
        autoFetchIntervalLabel->setObjectName("autoFetchIntervalLabel");
        autoFetchIntervalLabel->setEnabled(false);

        horizontalLayout->addWidget(autoFetchIntervalLabel);

        kcfg_AutoFetchInterval = new QSpinBox(groupBox_3);
        kcfg_AutoFetchInterval->setObjectName("kcfg_AutoFetchInterval");
        kcfg_AutoFetchInterval->setEnabled(false);
        kcfg_AutoFetchInterval->setMinimum(1);
        kcfg_AutoFetchInterval->setMaximum(3600);
        kcfg_AutoFetchInterval->setSingleStep(10);

        horizontalLayout->addWidget(kcfg_AutoFetchInterval);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        kcfg_PlasmaActivitySupport = new QCheckBox(groupBox_3);
        kcfg_PlasmaActivitySupport->setObjectName("kcfg_PlasmaActivitySupport");

        verticalLayout_3->addWidget(kcfg_PlasmaActivitySupport);


        verticalLayout_4->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(Akregator__SettingsGeneral);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_MarkAllFeedsReadOnStartup = new QCheckBox(groupBox_2);
        kcfg_MarkAllFeedsReadOnStartup->setObjectName("kcfg_MarkAllFeedsReadOnStartup");

        verticalLayout_2->addWidget(kcfg_MarkAllFeedsReadOnStartup);

        kcfg_FetchOnStartup = new QCheckBox(groupBox_2);
        kcfg_FetchOnStartup->setObjectName("kcfg_FetchOnStartup");

        verticalLayout_2->addWidget(kcfg_FetchOnStartup);


        verticalLayout_4->addWidget(groupBox_2);

        groupBox = new QGroupBox(Akregator__SettingsGeneral);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_UseHTMLCache = new QCheckBox(groupBox);
        kcfg_UseHTMLCache->setObjectName("kcfg_UseHTMLCache");

        verticalLayout->addWidget(kcfg_UseHTMLCache);


        verticalLayout_4->addWidget(groupBox);

        spacerItem1 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(spacerItem1);

        verticalLayout_4->setStretch(3, 1);

        retranslateUi(Akregator__SettingsGeneral);

        QMetaObject::connectSlotsByName(Akregator__SettingsGeneral);
    } // setupUi

    void retranslateUi(QWidget *Akregator__SettingsGeneral)
    {
        groupBox_3->setTitle(tr2i18n("Global", "global settings"));
        kcfg_ShowTrayIcon->setText(tr2i18n("Show tra&y icon", nullptr));
        kcfg_ShowUnreadInTaskbar->setText(tr2i18n("Show unread articles in Taskbar", nullptr));
        kcfg_HideReadFeeds->setText(tr2i18n("Hide feeds with no unread articles", nullptr));
        kcfg_LockFeedsInPlace->setText(tr2i18n("Lock feeds in place", nullptr));
        kcfg_AutoExpandFolders->setText(tr2i18n("Auto-expand folders containing unread articles", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_UseNotifications->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_UseNotifications->setWhatsThis(tr2i18n("Select this if you want to get notified when there are new articles.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_UseNotifications->setText(tr2i18n("Use &notifications for all feeds", nullptr));
        kcfg_DisableSaveAsNotification->setText(tr2i18n("Disable save as finished notification", nullptr));
        kcfg_UseIntervalFetch->setText(tr2i18n("&Use interval fetching", nullptr));
        autoFetchIntervalLabel->setText(tr2i18n("Fetch feeds every:", nullptr));
        kcfg_PlasmaActivitySupport->setText(tr2i18n("Enable Plasma Activities integration", nullptr));
        groupBox_2->setTitle(tr2i18n("Startup", nullptr));
        kcfg_MarkAllFeedsReadOnStartup->setText(tr2i18n("Mark &all feeds as read on startup", nullptr));
        kcfg_FetchOnStartup->setText(tr2i18n("Fetch all fee&ds on startup", nullptr));
        groupBox->setTitle(tr2i18n("Network", nullptr));
        kcfg_UseHTMLCache->setText(tr2i18n("Use the &browser cache (less network traffic)", nullptr));
        (void)Akregator__SettingsGeneral;
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class SettingsGeneral: public Ui_SettingsGeneral {};
} // namespace Ui
} // namespace Akregator

#endif // SETTINGS_GENERAL_H

