/****************************************************************************
** Meta object code from reading C++ file 'feedlist.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/feed/feedlist.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'feedlist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator8FeedListE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::FeedList::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator8FeedListE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::FeedList",
        "signalDestroyed",
        "",
        "Akregator::FeedList*",
        "signalNodeAdded",
        "Akregator::TreeNode*",
        "signalNodeRemoved",
        "signalAboutToRemoveNode",
        "signalNodeChanged",
        "fetchStarted",
        "Akregator::Feed*",
        "fetched",
        "fetchError",
        "fetchDiscovery",
        "fetchAborted",
        "unreadCountChanged",
        "unread",
        "slotNodeDestroyed",
        "node",
        "slotNodeAdded",
        "slotNodeRemoved",
        "Akregator::Folder*",
        "parent",
        "rootNodeChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalDestroyed'
        QtMocHelpers::SignalData<void(Akregator::FeedList *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalNodeAdded'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'signalNodeRemoved'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'signalAboutToRemoveNode'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'signalNodeChanged'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'fetchStarted'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'fetched'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'fetchError'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'fetchDiscovery'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'fetchAborted'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'unreadCountChanged'
        QtMocHelpers::SignalData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'slotNodeDestroyed'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 18 },
        }}),
        // Slot 'slotNodeAdded'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 18 },
        }}),
        // Slot 'slotNodeRemoved'
        QtMocHelpers::SlotData<void(Akregator::Folder *, Akregator::TreeNode *)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 5, 18 },
        }}),
        // Slot 'rootNodeChanged'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FeedList, qt_meta_tag_ZN9Akregator8FeedListE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::FeedList::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8FeedListE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8FeedListE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator8FeedListE_t>.metaTypes,
    nullptr
} };

void Akregator::FeedList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FeedList *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalDestroyed((*reinterpret_cast<std::add_pointer_t<Akregator::FeedList*>>(_a[1]))); break;
        case 1: _t->signalNodeAdded((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 2: _t->signalNodeRemoved((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 3: _t->signalAboutToRemoveNode((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 4: _t->signalNodeChanged((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 5: _t->fetchStarted((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 6: _t->fetched((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 7: _t->fetchError((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 8: _t->fetchDiscovery((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 9: _t->fetchAborted((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 10: _t->unreadCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->slotNodeDestroyed((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 12: _t->slotNodeAdded((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 13: _t->slotNodeRemoved((*reinterpret_cast<std::add_pointer_t<Akregator::Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[2]))); break;
        case 14: _t->rootNodeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::FeedList* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::FeedList * )>(_a, &FeedList::signalDestroyed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::TreeNode * )>(_a, &FeedList::signalNodeAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::TreeNode * )>(_a, &FeedList::signalNodeRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::TreeNode * )>(_a, &FeedList::signalAboutToRemoveNode, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::TreeNode * )>(_a, &FeedList::signalNodeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::Feed * )>(_a, &FeedList::fetchStarted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::Feed * )>(_a, &FeedList::fetched, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::Feed * )>(_a, &FeedList::fetchError, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::Feed * )>(_a, &FeedList::fetchDiscovery, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(Akregator::Feed * )>(_a, &FeedList::fetchAborted, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (FeedList::*)(int )>(_a, &FeedList::unreadCountChanged, 10))
            return;
    }
}

const QMetaObject *Akregator::FeedList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::FeedList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8FeedListE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Akregator::FeedList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Akregator::FeedList::signalDestroyed(Akregator::FeedList * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::FeedList::signalNodeAdded(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::FeedList::signalNodeRemoved(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::FeedList::signalAboutToRemoveNode(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Akregator::FeedList::signalNodeChanged(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Akregator::FeedList::fetchStarted(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Akregator::FeedList::fetched(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Akregator::FeedList::fetchError(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Akregator::FeedList::fetchDiscovery(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Akregator::FeedList::fetchAborted(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Akregator::FeedList::unreadCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
