/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.akregator.part.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef PARTINTERFACE_H
#define PARTINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.akregator.part
 */
class OrgKdeAkregatorPartInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.akregator.part"; }

public:
    OrgKdeAkregatorPartInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeAkregatorPartInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> addFeed()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("addFeed"), argumentList);
    }

    inline QDBusPendingReply<> addFeedsToGroup(const QStringList &lst, const QString &feedname)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(lst) << QVariant::fromValue(feedname);
        return asyncCallWithArgumentList(QStringLiteral("addFeedsToGroup"), argumentList);
    }

    inline QDBusPendingReply<> exportFile(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QStringLiteral("exportFile"), argumentList);
    }

    inline QDBusPendingReply<> fetchAllFeeds()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("fetchAllFeeds"), argumentList);
    }

    inline QDBusPendingReply<> fetchFeedUrl(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QStringLiteral("fetchFeedUrl"), argumentList);
    }

    inline QDBusPendingReply<bool> handleCommandLine(const QStringList &args)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(args);
        return asyncCallWithArgumentList(QStringLiteral("handleCommandLine"), argumentList);
    }

    inline QDBusPendingReply<> openStandardFeedList()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("openStandardFeedList"), argumentList);
    }

    inline QDBusPendingReply<> saveSettings()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("saveSettings"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace kde {
    namespace akregator {
      using part = ::OrgKdeAkregatorPartInterface;
    }
  }
}

#endif
