/********************************************************************************
** Form generated from reading UI file 'settings_advancedbase.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_ADVANCEDBASE_H
#define UI_SETTINGS_ADVANCEDBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace Akregator {

class Ui_SettingsAdvancedBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QCheckBox *kcfg_UseMarkReadDelay;
    QSpinBox *kcfg_MarkReadDelay;
    QSpacerItem *spacerItem;
    QCheckBox *kcfg_ResetQuickFilterOnNodeChange;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Akregator__SettingsAdvancedBase)
    {
        if (Akregator__SettingsAdvancedBase->objectName().isEmpty())
            Akregator__SettingsAdvancedBase->setObjectName("Akregator__SettingsAdvancedBase");
        Akregator__SettingsAdvancedBase->resize(411, 207);
        verticalLayout_2 = new QVBoxLayout(Akregator__SettingsAdvancedBase);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(Akregator__SettingsAdvancedBase);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_UseMarkReadDelay = new QCheckBox(groupBox);
        kcfg_UseMarkReadDelay->setObjectName("kcfg_UseMarkReadDelay");
        kcfg_UseMarkReadDelay->setChecked(true);

        horizontalLayout->addWidget(kcfg_UseMarkReadDelay);

        kcfg_MarkReadDelay = new QSpinBox(groupBox);
        kcfg_MarkReadDelay->setObjectName("kcfg_MarkReadDelay");
        kcfg_MarkReadDelay->setMinimumSize(QSize(130, 0));

        horizontalLayout->addWidget(kcfg_MarkReadDelay);

        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem);


        verticalLayout->addLayout(horizontalLayout);

        kcfg_ResetQuickFilterOnNodeChange = new QCheckBox(groupBox);
        kcfg_ResetQuickFilterOnNodeChange->setObjectName("kcfg_ResetQuickFilterOnNodeChange");
        kcfg_ResetQuickFilterOnNodeChange->setChecked(true);

        verticalLayout->addWidget(kcfg_ResetQuickFilterOnNodeChange);


        verticalLayout_2->addWidget(groupBox);

        spacerItem1 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacerItem1);

        verticalLayout_2->setStretch(1, 1);

        retranslateUi(Akregator__SettingsAdvancedBase);

        QMetaObject::connectSlotsByName(Akregator__SettingsAdvancedBase);
    } // setupUi

    void retranslateUi(QWidget *Akregator__SettingsAdvancedBase)
    {
        groupBox->setTitle(tr2i18n("Article List", nullptr));
        kcfg_UseMarkReadDelay->setText(tr2i18n("Mar&k selected article read after", nullptr));
        kcfg_ResetQuickFilterOnNodeChange->setText(tr2i18n("Reset search bar when changing feeds", nullptr));
        (void)Akregator__SettingsAdvancedBase;
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class SettingsAdvancedBase: public Ui_SettingsAdvancedBase {};
} // namespace Ui
} // namespace Akregator

#endif // SETTINGS_ADVANCEDBASE_H

