/****************************************************************************
** Meta object code from reading C++ file 'treenode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/treenode.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treenode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator8TreeNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::TreeNode::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator8TreeNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::TreeNode",
        "signalDestroyed",
        "",
        "Akregator::TreeNode*",
        "signalChanged",
        "signalArticlesAdded",
        "QList<Akregator::Article>",
        "guids",
        "signalArticlesUpdated",
        "signalArticlesRemoved",
        "slotAddToFetchQueue",
        "Akregator::FetchQueue*",
        "queue",
        "intervalFetchesOnly"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalDestroyed'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalChanged'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalArticlesAdded'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'signalArticlesUpdated'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'signalArticlesRemoved'
        QtMocHelpers::SignalData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotAddToFetchQueue'
        QtMocHelpers::SlotData<void(Akregator::FetchQueue *, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotAddToFetchQueue'
        QtMocHelpers::SlotData<void(Akregator::FetchQueue *)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreeNode, qt_meta_tag_ZN9Akregator8TreeNodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::TreeNode::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8TreeNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8TreeNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator8TreeNodeE_t>.metaTypes,
    nullptr
} };

void Akregator::TreeNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreeNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalDestroyed((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 1: _t->signalChanged((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 2: _t->signalArticlesAdded((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        case 3: _t->signalArticlesUpdated((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        case 4: _t->signalArticlesRemoved((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        case 5: _t->slotAddToFetchQueue((*reinterpret_cast<std::add_pointer_t<Akregator::FetchQueue*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->slotAddToFetchQueue((*reinterpret_cast<std::add_pointer_t<Akregator::FetchQueue*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreeNode::*)(Akregator::TreeNode * )>(_a, &TreeNode::signalDestroyed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeNode::*)(Akregator::TreeNode * )>(_a, &TreeNode::signalChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeNode::*)(Akregator::TreeNode * , const QList<Akregator::Article> & )>(_a, &TreeNode::signalArticlesAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeNode::*)(Akregator::TreeNode * , const QList<Akregator::Article> & )>(_a, &TreeNode::signalArticlesUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreeNode::*)(Akregator::TreeNode * , const QList<Akregator::Article> & )>(_a, &TreeNode::signalArticlesRemoved, 4))
            return;
    }
}

const QMetaObject *Akregator::TreeNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::TreeNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator8TreeNodeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Akregator::TreeNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Akregator::TreeNode::signalDestroyed(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::TreeNode::signalChanged(Akregator::TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::TreeNode::signalArticlesAdded(Akregator::TreeNode * _t1, const QList<Akregator::Article> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Akregator::TreeNode::signalArticlesUpdated(Akregator::TreeNode * _t1, const QList<Akregator::Article> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Akregator::TreeNode::signalArticlesRemoved(Akregator::TreeNode * _t1, const QList<Akregator::Article> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
