/****************************************************************************
** Meta object code from reading C++ file 'feed.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/feed/feed.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'feed.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator4FeedE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::Feed::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator4FeedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::Feed",
        "fetchStarted",
        "",
        "Akregator::Feed*",
        "fetched",
        "fetchError",
        "fetchDiscovery",
        "fetchAborted",
        "fetch",
        "followDiscovery",
        "slotAbortFetch",
        "slotAddToFetchQueue",
        "Akregator::FetchQueue*",
        "queue",
        "intervalFetchOnly",
        "slotAddFeedIconListener",
        "fetchCompleted",
        "Syndication::Loader*",
        "loader",
        "Syndication::FeedPtr",
        "doc",
        "Syndication::ErrorCode",
        "errorCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fetchStarted'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'fetched'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'fetchError'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'fetchDiscovery'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'fetchAborted'
        QtMocHelpers::SignalData<void(Akregator::Feed *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'fetch'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'fetch'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotAbortFetch'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddToFetchQueue'
        QtMocHelpers::SlotData<void(Akregator::FetchQueue *, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotAddToFetchQueue'
        QtMocHelpers::SlotData<void(Akregator::FetchQueue *)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotAddFeedIconListener'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchCompleted'
        QtMocHelpers::SlotData<void(Syndication::Loader *, Syndication::FeedPtr, Syndication::ErrorCode)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Feed, qt_meta_tag_ZN9Akregator4FeedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::Feed::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4FeedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4FeedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator4FeedE_t>.metaTypes,
    nullptr
} };

void Akregator::Feed::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Feed *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fetchStarted((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 1: _t->fetched((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 2: _t->fetchError((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 3: _t->fetchDiscovery((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 4: _t->fetchAborted((*reinterpret_cast<std::add_pointer_t<Akregator::Feed*>>(_a[1]))); break;
        case 5: _t->fetch((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fetch(); break;
        case 7: _t->slotAbortFetch(); break;
        case 8: _t->slotAddToFetchQueue((*reinterpret_cast<std::add_pointer_t<Akregator::FetchQueue*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->slotAddToFetchQueue((*reinterpret_cast<std::add_pointer_t<Akregator::FetchQueue*>>(_a[1]))); break;
        case 10: _t->slotAddFeedIconListener(); break;
        case 11: _t->fetchCompleted((*reinterpret_cast<std::add_pointer_t<Syndication::Loader*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Syndication::FeedPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Syndication::ErrorCode>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Feed* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Feed* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Feed* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Feed* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Feed* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Syndication::Loader* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Feed::*)(Akregator::Feed * )>(_a, &Feed::fetchStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Feed::*)(Akregator::Feed * )>(_a, &Feed::fetched, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Feed::*)(Akregator::Feed * )>(_a, &Feed::fetchError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Feed::*)(Akregator::Feed * )>(_a, &Feed::fetchDiscovery, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Feed::*)(Akregator::Feed * )>(_a, &Feed::fetchAborted, 4))
            return;
    }
}

const QMetaObject *Akregator::Feed::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::Feed::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator4FeedE_t>.strings))
        return static_cast<void*>(this);
    return TreeNode::qt_metacast(_clname);
}

int Akregator::Feed::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void Akregator::Feed::fetchStarted(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::Feed::fetched(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::Feed::fetchError(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::Feed::fetchDiscovery(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Akregator::Feed::fetchAborted(Akregator::Feed * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
