/****************************************************************************
** Meta object code from reading C++ file 'tabwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/tabwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator9TabWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::TabWidget::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator9TabWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::TabWidget",
        "signalCurrentFrameChanged",
        "",
        "signalRemoveFrameRequest",
        "signalOpenUrlRequest",
        "Akregator::OpenUrlRequest&",
        "signalZoomChangedInFrame",
        "currentFrame",
        "value",
        "signalPrintInFrame",
        "signalCopyInFrame",
        "signalPrintPreviewInFrame",
        "signalTextToSpeechInFrame",
        "signalFindTextInFrame",
        "signalMute",
        "slotWebPageMutedOrAudibleChanged",
        "Akregator::Frame*",
        "frame",
        "isAudioMuted",
        "wasRecentlyAudible",
        "slotCopy",
        "slotZoomChanged",
        "slotSetTitle",
        "title",
        "slotSetIcon",
        "QIcon",
        "icon",
        "slotSettingsChanged",
        "slotNextTab",
        "slotPreviousTab",
        "slotRemoveCurrentFrame",
        "slotAddFrame",
        "f",
        "slotRemoveFrame",
        "frameId",
        "slotSelectFrame",
        "slotReloadAllTabs",
        "slotReloadTab",
        "slotActivateTab",
        "slotDetachTab",
        "index",
        "slotCopyTabAddress",
        "slotCloseTab",
        "slotPrint",
        "slotPrintPreview",
        "slotFindTextInHtml",
        "slotTextToSpeech",
        "slotUnMute",
        "slotMute",
        "slotCloseRequest",
        "slotTabChanged",
        "slotCloseAllTab",
        "slotTabContextMenuRequest",
        "QPoint",
        "pos",
        "slotCloseAllTabExcept"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalCurrentFrameChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'signalRemoveFrameRequest'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'signalOpenUrlRequest'
        QtMocHelpers::SignalData<void(Akregator::OpenUrlRequest &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'signalZoomChangedInFrame'
        QtMocHelpers::SignalData<void(int, qreal)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QReal, 8 },
        }}),
        // Signal 'signalPrintInFrame'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'signalCopyInFrame'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'signalPrintPreviewInFrame'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'signalTextToSpeechInFrame'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'signalFindTextInFrame'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'signalMute'
        QtMocHelpers::SignalData<void(int, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotWebPageMutedOrAudibleChanged'
        QtMocHelpers::SlotData<void(Akregator::Frame *, bool, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Bool, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'slotCopy'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomChanged'
        QtMocHelpers::SlotData<void(qreal)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 8 },
        }}),
        // Slot 'slotSetTitle'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'slotSetIcon'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QIcon &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextTab'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPreviousTab'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveCurrentFrame'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddFrame'
        QtMocHelpers::SlotData<void(Akregator::Frame *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 32 },
        }}),
        // Slot 'slotRemoveFrame'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotSelectFrame'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotReloadAllTabs'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReloadTab'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotActivateTab'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDetachTab'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
        // Slot 'slotCopyTabAddress'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCloseTab'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrint'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrintPreview'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFindTextInHtml'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTextToSpeech'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUnMute'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMute'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCloseRequest'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
        // Slot 'slotTabChanged'
        QtMocHelpers::SlotData<void(int)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
        // Slot 'slotCloseAllTab'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTabContextMenuRequest'
        QtMocHelpers::SlotData<void(const QPoint &)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'slotCloseAllTabExcept'
        QtMocHelpers::SlotData<void(int)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TabWidget, qt_meta_tag_ZN9Akregator9TabWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::TabWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator9TabWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator9TabWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator9TabWidgetE_t>.metaTypes,
    nullptr
} };

void Akregator::TabWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TabWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalCurrentFrameChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->signalRemoveFrameRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->signalOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 3: _t->signalZoomChangedInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 4: _t->signalPrintInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->signalCopyInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->signalPrintPreviewInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->signalTextToSpeechInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->signalFindTextInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->signalMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->slotWebPageMutedOrAudibleChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 11: _t->slotCopy(); break;
        case 12: _t->slotZoomChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->slotSetTitle((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->slotSetIcon((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[2]))); break;
        case 15: _t->slotSettingsChanged(); break;
        case 16: _t->slotNextTab(); break;
        case 17: _t->slotPreviousTab(); break;
        case 18: _t->slotRemoveCurrentFrame(); break;
        case 19: _t->slotAddFrame((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 20: _t->slotRemoveFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->slotSelectFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->slotReloadAllTabs(); break;
        case 23: _t->slotReloadTab(); break;
        case 24: _t->slotActivateTab(); break;
        case 25: _t->slotDetachTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->slotCopyTabAddress(); break;
        case 27: _t->slotCloseTab(); break;
        case 28: _t->slotPrint(); break;
        case 29: _t->slotPrintPreview(); break;
        case 30: _t->slotFindTextInHtml(); break;
        case 31: _t->slotTextToSpeech(); break;
        case 32: _t->slotUnMute(); break;
        case 33: _t->slotMute(); break;
        case 34: _t->slotCloseRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->slotTabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->slotCloseAllTab(); break;
        case 37: _t->slotTabContextMenuRequest((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 38: _t->slotCloseAllTabExcept((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalCurrentFrameChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalRemoveFrameRequest, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(Akregator::OpenUrlRequest & )>(_a, &TabWidget::signalOpenUrlRequest, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int , qreal )>(_a, &TabWidget::signalZoomChangedInFrame, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalPrintInFrame, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalCopyInFrame, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalPrintPreviewInFrame, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalTextToSpeechInFrame, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int )>(_a, &TabWidget::signalFindTextInFrame, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabWidget::*)(int , bool )>(_a, &TabWidget::signalMute, 9))
            return;
    }
}

const QMetaObject *Akregator::TabWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::TabWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator9TabWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int Akregator::TabWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    return _id;
}

// SIGNAL 0
void Akregator::TabWidget::signalCurrentFrameChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::TabWidget::signalRemoveFrameRequest(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::TabWidget::signalOpenUrlRequest(Akregator::OpenUrlRequest & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::TabWidget::signalZoomChangedInFrame(int _t1, qreal _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Akregator::TabWidget::signalPrintInFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Akregator::TabWidget::signalCopyInFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Akregator::TabWidget::signalPrintPreviewInFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Akregator::TabWidget::signalTextToSpeechInFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Akregator::TabWidget::signalFindTextInFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Akregator::TabWidget::signalMute(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
