/****************************************************************************
** Meta object code from reading C++ file 'partadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "partadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'partadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11PartAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto PartAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN11PartAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PartAdaptor",
        "D-Bus Interface",
        "org.kde.akregator.part",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.akregator.part\">\n    <method name=\""
        "openStandardFeedList\"/>\n    <method name=\"fetchFeedUrl\">\n    "
        "  <arg direction=\"in\" type=\"s\" name=\"url\"/>\n    </method>\n"
        "    <method name=\"fetchAllFeeds\"/>\n    <method name=\"saveSetti"
        "ngs\"/>\n    <method name=\"addFeedsToGroup\">\n      <arg directi"
        "on=\"in\" type=\"as\" name=\"lst\"/>\n      <arg direction=\"in\" "
        "type=\"s\" name=\"feedname\"/>\n    </method>\n    <method name=\""
        "exportFile\">\n      <arg direction=\"in\" type=\"s\" name=\"url\""
        "/>\n    </method>\n    <method name=\"addFeed\"/>\n    <method nam"
        "e=\"handleCommandLine\">\n      <arg direction=\"in\" type=\"as\" "
        "name=\"args\"/>\n      <arg direction=\"out\" type=\"b\" name=\"re"
        "sult\"/>\n    </method>\n  </interface>\n",
        "addFeed",
        "",
        "addFeedsToGroup",
        "lst",
        "feedname",
        "exportFile",
        "url",
        "fetchAllFeeds",
        "fetchFeedUrl",
        "handleCommandLine",
        "args",
        "openStandardFeedList",
        "saveSettings"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addFeed'
        QtMocHelpers::SlotData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addFeedsToGroup'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 8 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'exportFile'
        QtMocHelpers::SlotData<void(const QString &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'fetchAllFeeds'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchFeedUrl'
        QtMocHelpers::SlotData<void(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'handleCommandLine'
        QtMocHelpers::SlotData<bool(const QStringList &)>(14, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QStringList, 15 },
        }}),
        // Slot 'openStandardFeedList'
        QtMocHelpers::SlotData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PartAdaptor, qt_meta_tag_ZN11PartAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PartAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PartAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PartAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11PartAdaptorE_t>.metaTypes,
    nullptr
} };

void PartAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PartAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addFeed(); break;
        case 1: _t->addFeedsToGroup((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->exportFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fetchAllFeeds(); break;
        case 4: _t->fetchFeedUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: { bool _r = _t->handleCommandLine((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->openStandardFeedList(); break;
        case 7: _t->saveSettings(); break;
        default: ;
        }
    }
}

const QMetaObject *PartAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PartAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PartAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int PartAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
