/****************************************************************************
** Meta object code from reading C++ file 'webengineframe.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/frame/webengine/webengineframe.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'webengineframe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator14WebEngineFrameE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::WebEngineFrame::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::WebEngineFrame",
        "signalIconChanged",
        "",
        "Akregator::Frame*",
        "QIcon",
        "icon",
        "webPageMutedOrAudibleChanged",
        "isAudioMuted",
        "wasRecentlyAudible",
        "slotReload",
        "slotStop",
        "slotZoomChangeInFrame",
        "frameId",
        "value",
        "slotCopyInFrame",
        "slotPrintInFrame",
        "slotPrintPreviewInFrame",
        "slotFindTextInFrame",
        "slotTextToSpeechInFrame",
        "slotMute",
        "mute",
        "slotTitleChanged",
        "title",
        "slotProgressChanged",
        "progress",
        "slotLoadStarted",
        "slotLoadFinished",
        "slotWebPageMutedOrAudibleChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalIconChanged'
        QtMocHelpers::SignalData<void(Akregator::Frame *, const QIcon &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'webPageMutedOrAudibleChanged'
        QtMocHelpers::SignalData<void(Akregator::Frame *, bool, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotReload'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStop'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomChangeInFrame'
        QtMocHelpers::SlotData<void(int, qreal)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 }, { QMetaType::QReal, 13 },
        }}),
        // Slot 'slotCopyInFrame'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotPrintInFrame'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotPrintPreviewInFrame'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotFindTextInFrame'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotTextToSpeechInFrame'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'slotMute'
        QtMocHelpers::SlotData<void(int, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'slotTitleChanged'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'slotProgressChanged'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'slotLoadStarted'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadFinished'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWebPageMutedOrAudibleChanged'
        QtMocHelpers::SlotData<void(bool, bool)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { QMetaType::Bool, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WebEngineFrame, qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::WebEngineFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<Frame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>.metaTypes,
    nullptr
} };

void Akregator::WebEngineFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WebEngineFrame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalIconChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[2]))); break;
        case 1: _t->webPageMutedOrAudibleChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->slotReload(); break;
        case 3: _t->slotStop(); break;
        case 4: _t->slotZoomChangeInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2]))); break;
        case 5: _t->slotCopyInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotPrintInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->slotPrintPreviewInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->slotFindTextInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->slotTextToSpeechInFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->slotMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->slotTitleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotProgressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->slotLoadStarted(); break;
        case 14: _t->slotLoadFinished(); break;
        case 15: _t->slotWebPageMutedOrAudibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WebEngineFrame::*)(Akregator::Frame * , const QIcon & )>(_a, &WebEngineFrame::signalIconChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WebEngineFrame::*)(Akregator::Frame * , bool , bool )>(_a, &WebEngineFrame::webPageMutedOrAudibleChanged, 1))
            return;
    }
}

const QMetaObject *Akregator::WebEngineFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::WebEngineFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator14WebEngineFrameE_t>.strings))
        return static_cast<void*>(this);
    return Frame::qt_metacast(_clname);
}

int Akregator::WebEngineFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Frame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Akregator::WebEngineFrame::signalIconChanged(Akregator::Frame * _t1, const QIcon & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Akregator::WebEngineFrame::webPageMutedOrAudibleChanged(Akregator::Frame * _t1, bool _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
