/********************************************************************************
** Form generated from reading UI file 'feedpropertieswidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FEEDPROPERTIESWIDGETBASE_H
#define UI_FEEDPROPERTIESWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace Akregator {

class Ui_FeedPropertiesWidgetBase
{
public:
    QGridLayout *gridLayout;
    QTabWidget *tabWidget;
    QWidget *tab1;
    QGridLayout *gridLayout1;
    QCheckBox *cb_updateInterval;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QLabel *updateLabel;
    QSpinBox *updateSpinBox;
    QComboBox *updateComboBox;
    QCheckBox *checkBox_useNotification;
    QGridLayout *gridLayout2;
    QLabel *textLabel3;
    QLabel *textLabel2;
    QLineEdit *urlEdit;
    QLineEdit *feedNameEdit;
    QLineEdit *commentEdit;
    QLabel *label;
    QSpacerItem *spacerItem1;
    QWidget *tab2;
    QGridLayout *gridLayout3;
    QRadioButton *rb_globalDefault;
    QRadioButton *rb_disableArchiving;
    QHBoxLayout *hboxLayout1;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout2;
    QRadioButton *rb_limitArticleNumber;
    QSpinBox *sb_maxArticleNumber;
    QHBoxLayout *hboxLayout3;
    QRadioButton *rb_limitArticleAge;
    QSpinBox *sb_maxArticleAge;
    QSpacerItem *spacerItem2;
    QSpacerItem *spacerItem3;
    QRadioButton *rb_keepAllArticles;
    QWidget *tab3;
    QGridLayout *gridLayout4;
    QSpacerItem *spacerItem4;
    QVBoxLayout *vboxLayout1;
    QCheckBox *checkBox_loadWebsite;
    QCheckBox *checkBox_markRead;

    void setupUi(QWidget *Akregator__FeedPropertiesWidgetBase)
    {
        if (Akregator__FeedPropertiesWidgetBase->objectName().isEmpty())
            Akregator__FeedPropertiesWidgetBase->setObjectName("Akregator__FeedPropertiesWidgetBase");
        Akregator__FeedPropertiesWidgetBase->resize(450, 304);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Akregator__FeedPropertiesWidgetBase->sizePolicy().hasHeightForWidth());
        Akregator__FeedPropertiesWidgetBase->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(Akregator__FeedPropertiesWidgetBase);
#ifndef UI_Q_OS_MACOS
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        tabWidget = new QTabWidget(Akregator__FeedPropertiesWidgetBase);
        tabWidget->setObjectName("tabWidget");
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        tabWidget->setDocumentMode(true);
        tab1 = new QWidget();
        tab1->setObjectName("tab1");
        gridLayout1 = new QGridLayout(tab1);
#ifndef UI_Q_OS_MACOS
        gridLayout1->setSpacing(6);
#endif
        gridLayout1->setContentsMargins(8, 8, 8, 8);
        gridLayout1->setObjectName("gridLayout1");
        cb_updateInterval = new QCheckBox(tab1);
        cb_updateInterval->setObjectName("cb_updateInterval");

        gridLayout1->addWidget(cb_updateInterval, 1, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(31, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        updateLabel = new QLabel(tab1);
        updateLabel->setObjectName("updateLabel");
        updateLabel->setEnabled(false);

        hboxLayout->addWidget(updateLabel);

        updateSpinBox = new QSpinBox(tab1);
        updateSpinBox->setObjectName("updateSpinBox");
        updateSpinBox->setEnabled(false);
        updateSpinBox->setMinimum(1);

        hboxLayout->addWidget(updateSpinBox);

        updateComboBox = new QComboBox(tab1);
        updateComboBox->setObjectName("updateComboBox");
        updateComboBox->setEnabled(false);

        hboxLayout->addWidget(updateComboBox);


        gridLayout1->addLayout(hboxLayout, 2, 0, 1, 1);

        checkBox_useNotification = new QCheckBox(tab1);
        checkBox_useNotification->setObjectName("checkBox_useNotification");

        gridLayout1->addWidget(checkBox_useNotification, 3, 0, 1, 1);

        gridLayout2 = new QGridLayout();
#ifndef UI_Q_OS_MACOS
        gridLayout2->setSpacing(6);
#endif
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout2->setObjectName("gridLayout2");
        textLabel3 = new QLabel(tab1);
        textLabel3->setObjectName("textLabel3");

        gridLayout2->addWidget(textLabel3, 1, 0, 1, 1);

        textLabel2 = new QLabel(tab1);
        textLabel2->setObjectName("textLabel2");

        gridLayout2->addWidget(textLabel2, 0, 0, 1, 1);

        urlEdit = new QLineEdit(tab1);
        urlEdit->setObjectName("urlEdit");

        gridLayout2->addWidget(urlEdit, 1, 1, 1, 1);

        feedNameEdit = new QLineEdit(tab1);
        feedNameEdit->setObjectName("feedNameEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(feedNameEdit->sizePolicy().hasHeightForWidth());
        feedNameEdit->setSizePolicy(sizePolicy1);

        gridLayout2->addWidget(feedNameEdit, 0, 1, 1, 1);

        commentEdit = new QLineEdit(tab1);
        commentEdit->setObjectName("commentEdit");

        gridLayout2->addWidget(commentEdit, 2, 1, 1, 1);

        label = new QLabel(tab1);
        label->setObjectName("label");

        gridLayout2->addWidget(label, 2, 0, 1, 1);


        gridLayout1->addLayout(gridLayout2, 0, 0, 1, 1);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem1, 4, 0, 1, 1);

        tabWidget->addTab(tab1, QString());
        tab2 = new QWidget();
        tab2->setObjectName("tab2");
        gridLayout3 = new QGridLayout(tab2);
#ifndef UI_Q_OS_MACOS
        gridLayout3->setSpacing(6);
#endif
        gridLayout3->setContentsMargins(8, 8, 8, 8);
        gridLayout3->setObjectName("gridLayout3");
        rb_globalDefault = new QRadioButton(tab2);
        rb_globalDefault->setObjectName("rb_globalDefault");
        rb_globalDefault->setChecked(true);

        gridLayout3->addWidget(rb_globalDefault, 0, 0, 1, 1);

        rb_disableArchiving = new QRadioButton(tab2);
        rb_disableArchiving->setObjectName("rb_disableArchiving");

        gridLayout3->addWidget(rb_disableArchiving, 3, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        vboxLayout = new QVBoxLayout();
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout2 = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout2->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout2->setObjectName("hboxLayout2");
        rb_limitArticleNumber = new QRadioButton(tab2);
        rb_limitArticleNumber->setObjectName("rb_limitArticleNumber");

        hboxLayout2->addWidget(rb_limitArticleNumber);

        sb_maxArticleNumber = new QSpinBox(tab2);
        sb_maxArticleNumber->setObjectName("sb_maxArticleNumber");
        sb_maxArticleNumber->setEnabled(false);
        sb_maxArticleNumber->setMaximum(999999999);
        sb_maxArticleNumber->setValue(1000);

        hboxLayout2->addWidget(sb_maxArticleNumber);


        vboxLayout->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName("hboxLayout3");
        rb_limitArticleAge = new QRadioButton(tab2);
        rb_limitArticleAge->setObjectName("rb_limitArticleAge");

        hboxLayout3->addWidget(rb_limitArticleAge);

        sb_maxArticleAge = new QSpinBox(tab2);
        sb_maxArticleAge->setObjectName("sb_maxArticleAge");
        sb_maxArticleAge->setEnabled(false);
        sb_maxArticleAge->setMaximum(999999999);
        sb_maxArticleAge->setValue(30);

        hboxLayout3->addWidget(sb_maxArticleAge);


        vboxLayout->addLayout(hboxLayout3);


        hboxLayout1->addLayout(vboxLayout);

        spacerItem2 = new QSpacerItem(99, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem2);


        gridLayout3->addLayout(hboxLayout1, 2, 0, 1, 1);

        spacerItem3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout3->addItem(spacerItem3, 4, 0, 1, 1);

        rb_keepAllArticles = new QRadioButton(tab2);
        rb_keepAllArticles->setObjectName("rb_keepAllArticles");

        gridLayout3->addWidget(rb_keepAllArticles, 1, 0, 1, 1);

        tabWidget->addTab(tab2, QString());
        tab3 = new QWidget();
        tab3->setObjectName("tab3");
        gridLayout4 = new QGridLayout(tab3);
#ifndef UI_Q_OS_MACOS
        gridLayout4->setSpacing(6);
#endif
        gridLayout4->setContentsMargins(8, 8, 8, 8);
        gridLayout4->setObjectName("gridLayout4");
        spacerItem4 = new QSpacerItem(414, 141, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout4->addItem(spacerItem4, 1, 0, 1, 1);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        checkBox_loadWebsite = new QCheckBox(tab3);
        checkBox_loadWebsite->setObjectName("checkBox_loadWebsite");

        vboxLayout1->addWidget(checkBox_loadWebsite);

        checkBox_markRead = new QCheckBox(tab3);
        checkBox_markRead->setObjectName("checkBox_markRead");

        vboxLayout1->addWidget(checkBox_markRead);


        gridLayout4->addLayout(vboxLayout1, 0, 0, 1, 1);

        tabWidget->addTab(tab3, QString());

        gridLayout->addWidget(tabWidget, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        updateLabel->setBuddy(updateSpinBox);
        textLabel3->setBuddy(urlEdit);
        textLabel2->setBuddy(feedNameEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(feedNameEdit, urlEdit);
        QWidget::setTabOrder(urlEdit, commentEdit);
        QWidget::setTabOrder(commentEdit, cb_updateInterval);
        QWidget::setTabOrder(cb_updateInterval, updateSpinBox);
        QWidget::setTabOrder(updateSpinBox, updateComboBox);
        QWidget::setTabOrder(updateComboBox, checkBox_useNotification);
        QWidget::setTabOrder(checkBox_useNotification, rb_globalDefault);
        QWidget::setTabOrder(rb_globalDefault, rb_keepAllArticles);
        QWidget::setTabOrder(rb_keepAllArticles, rb_limitArticleNumber);
        QWidget::setTabOrder(rb_limitArticleNumber, sb_maxArticleNumber);
        QWidget::setTabOrder(sb_maxArticleNumber, rb_limitArticleAge);
        QWidget::setTabOrder(rb_limitArticleAge, sb_maxArticleAge);
        QWidget::setTabOrder(sb_maxArticleAge, rb_disableArchiving);
        QWidget::setTabOrder(rb_disableArchiving, checkBox_loadWebsite);
        QWidget::setTabOrder(checkBox_loadWebsite, checkBox_markRead);
        QWidget::setTabOrder(checkBox_markRead, tabWidget);

        retranslateUi(Akregator__FeedPropertiesWidgetBase);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Akregator__FeedPropertiesWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *Akregator__FeedPropertiesWidgetBase)
    {
        Akregator__FeedPropertiesWidgetBase->setWindowTitle(tr2i18n("Feed Properties", nullptr));
        cb_updateInterval->setText(tr2i18n("U&se a custom update interval", nullptr));
        updateLabel->setText(tr2i18n("Update ever&y:", nullptr));
        checkBox_useNotification->setText(tr2i18n("Notify when new articles arri&ve", nullptr));
        textLabel3->setText(tr2i18n("&URL:", nullptr));
        textLabel2->setText(tr2i18n("&Name:", nullptr));
#if QT_CONFIG(tooltip)
        feedNameEdit->setToolTip(tr2i18n("Display name of RSS column", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(tr2i18n("Comment:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab1), tr2i18n("&General", nullptr));
        rb_globalDefault->setText(tr2i18n("&Use default settings", nullptr));
        rb_disableArchiving->setText(tr2i18n("Di&sable archiving", nullptr));
        rb_limitArticleNumber->setText(tr2i18n("Limit archi&ve to:", nullptr));
        rb_limitArticleAge->setText(tr2i18n("&Delete articles older than:", nullptr));
        rb_keepAllArticles->setText(tr2i18n("&Keep all articles", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab2), tr2i18n("Ar&chive", nullptr));
        checkBox_loadWebsite->setText(tr2i18n("Load the &full website when reading articles", nullptr));
        checkBox_markRead->setText(tr2i18n("Mar&k articles as read when they arrive", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab3), tr2i18n("Adva&nced", nullptr));
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class FeedPropertiesWidgetBase: public Ui_FeedPropertiesWidgetBase {};
} // namespace Ui
} // namespace Akregator

#endif // FEEDPROPERTIESWIDGETBASE_H

