/********************************************************************************
** Form generated from reading UI file 'settings_browser.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_BROWSER_H
#define UI_SETTINGS_BROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

namespace Akregator {

class Ui_SettingsBrowser
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *LMBlabel;
    QComboBox *kcfg_LMBBehaviour;
    QLabel *MMBlabel;
    QComboBox *kcfg_MMBBehaviour;
    QGroupBox *externalBrowserGroupBox;
    QGridLayout *gridLayout;
    QLineEdit *kcfg_ExternalBrowserCustomCommand;
    QRadioButton *kcfg_ExternalBrowserUseCustomCommand;
    QRadioButton *kcfg_ExternalBrowserUseKdeDefault;
    QGroupBox *tabsGroupBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_AlwaysShowTabBar;
    QCheckBox *kcfg_CloseButtonOnTabs;
    QCheckBox *kcfg_NewWindowInTab;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *kcfg_CheckPhishingUrl;
    QCheckBox *kcfg_EnableJavascript;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *kcfg_AccessKeyEnabled;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Akregator__SettingsBrowser)
    {
        if (Akregator__SettingsBrowser->objectName().isEmpty())
            Akregator__SettingsBrowser->setObjectName("Akregator__SettingsBrowser");
        Akregator__SettingsBrowser->resize(474, 510);
        verticalLayout = new QVBoxLayout(Akregator__SettingsBrowser);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        LMBlabel = new QLabel(Akregator__SettingsBrowser);
        LMBlabel->setObjectName("LMBlabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, LMBlabel);

        kcfg_LMBBehaviour = new QComboBox(Akregator__SettingsBrowser);
        kcfg_LMBBehaviour->addItem(QString());
        kcfg_LMBBehaviour->addItem(QString());
        kcfg_LMBBehaviour->addItem(QString());
        kcfg_LMBBehaviour->setObjectName("kcfg_LMBBehaviour");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_LMBBehaviour);

        MMBlabel = new QLabel(Akregator__SettingsBrowser);
        MMBlabel->setObjectName("MMBlabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, MMBlabel);

        kcfg_MMBBehaviour = new QComboBox(Akregator__SettingsBrowser);
        kcfg_MMBBehaviour->addItem(QString());
        kcfg_MMBBehaviour->addItem(QString());
        kcfg_MMBBehaviour->addItem(QString());
        kcfg_MMBBehaviour->setObjectName("kcfg_MMBBehaviour");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_MMBBehaviour);


        verticalLayout->addLayout(formLayout);

        externalBrowserGroupBox = new QGroupBox(Akregator__SettingsBrowser);
        externalBrowserGroupBox->setObjectName("externalBrowserGroupBox");
        gridLayout = new QGridLayout(externalBrowserGroupBox);
        gridLayout->setObjectName("gridLayout");
        kcfg_ExternalBrowserCustomCommand = new QLineEdit(externalBrowserGroupBox);
        kcfg_ExternalBrowserCustomCommand->setObjectName("kcfg_ExternalBrowserCustomCommand");
        kcfg_ExternalBrowserCustomCommand->setEnabled(false);

        gridLayout->addWidget(kcfg_ExternalBrowserCustomCommand, 1, 1, 1, 1);

        kcfg_ExternalBrowserUseCustomCommand = new QRadioButton(externalBrowserGroupBox);
        kcfg_ExternalBrowserUseCustomCommand->setObjectName("kcfg_ExternalBrowserUseCustomCommand");

        gridLayout->addWidget(kcfg_ExternalBrowserUseCustomCommand, 1, 0, 1, 1);

        kcfg_ExternalBrowserUseKdeDefault = new QRadioButton(externalBrowserGroupBox);
        kcfg_ExternalBrowserUseKdeDefault->setObjectName("kcfg_ExternalBrowserUseKdeDefault");
        kcfg_ExternalBrowserUseKdeDefault->setChecked(true);

        gridLayout->addWidget(kcfg_ExternalBrowserUseKdeDefault, 0, 0, 1, 2);


        verticalLayout->addWidget(externalBrowserGroupBox);

        tabsGroupBox = new QGroupBox(Akregator__SettingsBrowser);
        tabsGroupBox->setObjectName("tabsGroupBox");
        verticalLayout_2 = new QVBoxLayout(tabsGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_AlwaysShowTabBar = new QCheckBox(tabsGroupBox);
        kcfg_AlwaysShowTabBar->setObjectName("kcfg_AlwaysShowTabBar");

        verticalLayout_2->addWidget(kcfg_AlwaysShowTabBar);

        kcfg_CloseButtonOnTabs = new QCheckBox(tabsGroupBox);
        kcfg_CloseButtonOnTabs->setObjectName("kcfg_CloseButtonOnTabs");

        verticalLayout_2->addWidget(kcfg_CloseButtonOnTabs);

        kcfg_NewWindowInTab = new QCheckBox(tabsGroupBox);
        kcfg_NewWindowInTab->setObjectName("kcfg_NewWindowInTab");

        verticalLayout_2->addWidget(kcfg_NewWindowInTab);


        verticalLayout->addWidget(tabsGroupBox);

        groupBox = new QGroupBox(Akregator__SettingsBrowser);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        kcfg_CheckPhishingUrl = new QCheckBox(groupBox);
        kcfg_CheckPhishingUrl->setObjectName("kcfg_CheckPhishingUrl");

        verticalLayout_3->addWidget(kcfg_CheckPhishingUrl);

        kcfg_EnableJavascript = new QCheckBox(groupBox);
        kcfg_EnableJavascript->setObjectName("kcfg_EnableJavascript");

        verticalLayout_3->addWidget(kcfg_EnableJavascript);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(Akregator__SettingsBrowser);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_4 = new QVBoxLayout(groupBox_2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        kcfg_AccessKeyEnabled = new QCheckBox(groupBox_2);
        kcfg_AccessKeyEnabled->setObjectName("kcfg_AccessKeyEnabled");

        verticalLayout_4->addWidget(kcfg_AccessKeyEnabled);


        verticalLayout->addWidget(groupBox_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(Akregator__SettingsBrowser);

        QMetaObject::connectSlotsByName(Akregator__SettingsBrowser);
    } // setupUi

    void retranslateUi(QWidget *Akregator__SettingsBrowser)
    {
        LMBlabel->setText(tr2i18n("Left mouse click:", nullptr));
        kcfg_LMBBehaviour->setItemText(0, tr2i18n("Open in Tab", nullptr));
        kcfg_LMBBehaviour->setItemText(1, tr2i18n("Open in Background Tab", nullptr));
        kcfg_LMBBehaviour->setItemText(2, tr2i18n("Open in External Browser", nullptr));

        MMBlabel->setText(tr2i18n("Middle mouse click:", nullptr));
        kcfg_MMBBehaviour->setItemText(0, tr2i18n("Open in Tab", nullptr));
        kcfg_MMBBehaviour->setItemText(1, tr2i18n("Open in Background Tab", nullptr));
        kcfg_MMBBehaviour->setItemText(2, tr2i18n("Open in External Browser", nullptr));

        externalBrowserGroupBox->setTitle(tr2i18n("External Browsing", nullptr));
        kcfg_ExternalBrowserCustomCommand->setText(tr2i18n("firefox %u", nullptr));
        kcfg_ExternalBrowserUseCustomCommand->setText(tr2i18n("Use &this command:", nullptr));
        kcfg_ExternalBrowserUseKdeDefault->setText(tr2i18n("Use default web &browser", nullptr));
        tabsGroupBox->setTitle(tr2i18n("Tabs", nullptr));
        kcfg_AlwaysShowTabBar->setText(tr2i18n("Always show the tab bar", nullptr));
        kcfg_CloseButtonOnTabs->setText(tr2i18n("Show close button on each tab", nullptr));
        kcfg_NewWindowInTab->setText(tr2i18n("Open links in new tab instead of in new window", nullptr));
        groupBox->setTitle(tr2i18n("Security", nullptr));
        kcfg_CheckPhishingUrl->setText(tr2i18n("Verify URLs using Google's Safe Browsing API", nullptr));
        kcfg_EnableJavascript->setText(tr2i18n("Enable JavaScript", nullptr));
        groupBox_2->setTitle(tr2i18n("Accessibility", nullptr));
        kcfg_AccessKeyEnabled->setText(tr2i18n("Enable Access Key activation with Ctrl key", nullptr));
        (void)Akregator__SettingsBrowser;
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class SettingsBrowser: public Ui_SettingsBrowser {};
} // namespace Ui
} // namespace Akregator

#endif // SETTINGS_BROWSER_H

