/****************************************************************************
** Meta object code from reading C++ file 'subscriptionlistview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/subscription/subscriptionlistview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'subscriptionlistview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::SubscriptionListView::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::SubscriptionListView",
        "userActionTakingPlace",
        "",
        "slotPrevFeed",
        "slotNextFeed",
        "slotPrevUnreadFeed",
        "slotNextUnreadFeed",
        "slotItemBegin",
        "slotItemEnd",
        "slotItemLeft",
        "slotItemRight",
        "slotItemUp",
        "slotItemDown",
        "slotSetHideReadFeeds",
        "setting",
        "slotSetLockFeedsInPlace",
        "slotSetAutoExpandFolders"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'userActionTakingPlace'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrevFeed'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextFeed'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrevUnreadFeed'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextUnreadFeed'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemBegin'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemEnd'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemLeft'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemRight'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemUp'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotItemDown'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetHideReadFeeds'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotSetLockFeedsInPlace'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotSetAutoExpandFolders'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SubscriptionListView, qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::SubscriptionListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>.metaTypes,
    nullptr
} };

void Akregator::SubscriptionListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SubscriptionListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->userActionTakingPlace(); break;
        case 1: _t->slotPrevFeed(); break;
        case 2: _t->slotNextFeed(); break;
        case 3: _t->slotPrevUnreadFeed(); break;
        case 4: _t->slotNextUnreadFeed(); break;
        case 5: _t->slotItemBegin(); break;
        case 6: _t->slotItemEnd(); break;
        case 7: _t->slotItemLeft(); break;
        case 8: _t->slotItemRight(); break;
        case 9: _t->slotItemUp(); break;
        case 10: _t->slotItemDown(); break;
        case 11: _t->slotSetHideReadFeeds((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotSetLockFeedsInPlace((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotSetAutoExpandFolders((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SubscriptionListView::*)()>(_a, &SubscriptionListView::userActionTakingPlace, 0))
            return;
    }
}

const QMetaObject *Akregator::SubscriptionListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::SubscriptionListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator20SubscriptionListViewE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int Akregator::SubscriptionListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Akregator::SubscriptionListView::userActionTakingPlace()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
