// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kcm_config_plugins_debug.h"


Q_LOGGING_CATEGORY(AKREGATOR_CONFIG_PLUGIN_LOG, "org.kde.pim.akregator_config_plugin", QtInfoMsg)

