/********************************************************************************
** Form generated from reading UI file 'addfeedwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADDFEEDWIDGETBASE_H
#define UI_ADDFEEDWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "ksqueezedtextlabel.h"

namespace Akregator {

class Ui_AddFeedWidgetBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *pixmapLabel1;
    QSpacerItem *spacerItem;
    QGridLayout *gridLayout;
    QLineEdit *urlEdit;
    QLabel *textLabel1;
    QLabel *textLabel3;
    KSqueezedTextLabel *statusLabel;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Akregator__AddFeedWidgetBase)
    {
        if (Akregator__AddFeedWidgetBase->objectName().isEmpty())
            Akregator__AddFeedWidgetBase->setObjectName("Akregator__AddFeedWidgetBase");
        Akregator__AddFeedWidgetBase->resize(567, 176);
        vboxLayout = new QVBoxLayout(Akregator__AddFeedWidgetBase);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        pixmapLabel1 = new QLabel(Akregator__AddFeedWidgetBase);
        pixmapLabel1->setObjectName("pixmapLabel1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pixmapLabel1->sizePolicy().hasHeightForWidth());
        pixmapLabel1->setSizePolicy(sizePolicy);
        pixmapLabel1->setScaledContents(false);

        hboxLayout->addWidget(pixmapLabel1);

        spacerItem = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        urlEdit = new QLineEdit(Akregator__AddFeedWidgetBase);
        urlEdit->setObjectName("urlEdit");
        urlEdit->setMinimumSize(QSize(200, 0));
        urlEdit->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(urlEdit, 1, 1, 1, 2);

        textLabel1 = new QLabel(Akregator__AddFeedWidgetBase);
        textLabel1->setObjectName("textLabel1");
        QFont font;
        font.setFamilies({QString::fromUtf8("Sans Serif")});
        font.setPointSize(12);
        font.setBold(false);
        font.setItalic(false);
        font.setUnderline(false);
        font.setStrikeOut(false);
        textLabel1->setFont(font);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 3);

        textLabel3 = new QLabel(Akregator__AddFeedWidgetBase);
        textLabel3->setObjectName("textLabel3");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(textLabel3, 1, 0, 1, 1);


        hboxLayout->addLayout(gridLayout);


        vboxLayout->addLayout(hboxLayout);

        statusLabel = new KSqueezedTextLabel(Akregator__AddFeedWidgetBase);
        statusLabel->setObjectName("statusLabel");
        statusLabel->setText(QString::fromUtf8("KSqueezedTextLabel"));

        vboxLayout->addWidget(statusLabel);

        spacerItem1 = new QSpacerItem(20, 50, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem1);

#if QT_CONFIG(shortcut)
        textLabel3->setBuddy(urlEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Akregator__AddFeedWidgetBase);

        QMetaObject::connectSlotsByName(Akregator__AddFeedWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *Akregator__AddFeedWidgetBase)
    {
        Akregator__AddFeedWidgetBase->setWindowTitle(tr2i18n("Add Feed", nullptr));
        textLabel1->setText(tr2i18n("Add New Source", nullptr));
        textLabel3->setText(tr2i18n("Feed &URL:", nullptr));
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class AddFeedWidgetBase: public Ui_AddFeedWidgetBase {};
} // namespace Ui
} // namespace Akregator

#endif // ADDFEEDWIDGETBASE_H

