/****************************************************************************
** Meta object code from reading C++ file 'articleviewerwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/articleviewerwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'articleviewerwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::ArticleViewerWidget::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::ArticleViewerWidget",
        "signalOpenUrlRequest",
        "",
        "Akregator::OpenUrlRequest&",
        "showStatusBarMessage",
        "msg",
        "selectionChanged",
        "slotPrint",
        "setFilters",
        "std::vector<QSharedPointer<const Akregator::Filters::AbstractMatch"
        "er>>",
        "filters",
        "slotUpdateCombinedView",
        "slotClear",
        "slotShowSummary",
        "Akregator::TreeNode*",
        "node",
        "slotPrintPreview",
        "slotCopy",
        "slotZoomChangeInFrame",
        "value",
        "slotSelectionChanged",
        "slotArticlesListed",
        "KJob*",
        "job",
        "slotArticlesUpdated",
        "QList<Akregator::Article>",
        "list",
        "slotArticlesAdded",
        "slotArticlesRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalOpenUrlRequest'
        QtMocHelpers::SignalData<void(Akregator::OpenUrlRequest &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'showStatusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrint'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFilters'
        QtMocHelpers::SlotData<void(const std::vector<QSharedPointer<const Akregator::Filters::AbstractMatcher>> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotUpdateCombinedView'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotClear'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowSummary'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotPrintPreview'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCopy'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotZoomChangeInFrame'
        QtMocHelpers::SlotData<void(qreal)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 19 },
        }}),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotArticlesListed'
        QtMocHelpers::SlotData<void(KJob *)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotArticlesUpdated'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotArticlesAdded'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(27, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slotArticlesRemoved'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *, const QList<Akregator::Article> &)>(28, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 25, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ArticleViewerWidget, qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::ArticleViewerWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>.metaTypes,
    nullptr
} };

void Akregator::ArticleViewerWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArticleViewerWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 1: _t->showStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->selectionChanged(); break;
        case 3: _t->slotPrint(); break;
        case 4: _t->setFilters((*reinterpret_cast<std::add_pointer_t<std::vector<QSharedPointer<const Akregator::Filters::AbstractMatcher>>>>(_a[1]))); break;
        case 5: _t->slotUpdateCombinedView(); break;
        case 6: _t->slotClear(); break;
        case 7: _t->slotShowSummary((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 8: _t->slotPrintPreview(); break;
        case 9: _t->slotCopy(); break;
        case 10: _t->slotZoomChangeInFrame((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 11: _t->slotSelectionChanged(); break;
        case 12: _t->slotArticlesListed((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 13: _t->slotArticlesUpdated((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        case 14: _t->slotArticlesAdded((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        case 15: _t->slotArticlesRemoved((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWidget::*)(Akregator::OpenUrlRequest & )>(_a, &ArticleViewerWidget::signalOpenUrlRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWidget::*)(const QString & )>(_a, &ArticleViewerWidget::showStatusBarMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWidget::*)()>(_a, &ArticleViewerWidget::selectionChanged, 2))
            return;
    }
}

const QMetaObject *Akregator::ArticleViewerWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::ArticleViewerWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator19ArticleViewerWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Akregator::ArticleViewerWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Akregator::ArticleViewerWidget::signalOpenUrlRequest(Akregator::OpenUrlRequest & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::ArticleViewerWidget::showStatusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::ArticleViewerWidget::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
