// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "akregator_plasma_activities_debug.h"


Q_LOGGING_CATEGORY(AKREGATOR_PLASMA_ACTIVITIES_LOG, "org.kde.pim.akregator.plasma.activities", QtInfoMsg)

