/********************************************************************************
** Form generated from reading UI file 'settings_appearance.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_APPEARANCE_H
#define UI_SETTINGS_APPEARANCE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorbutton.h"

namespace Akregator {

class Ui_SettingsAppearance
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout;
    QFormLayout *formLayout_3;
    QLabel *lbl_unreadArticles;
    KColorButton *kcfg_ColorUnreadArticles;
    QLabel *lbl_newArticles;
    KColorButton *kcfg_ColorNewArticles;
    QCheckBox *kcfg_UseCustomColors;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox_2;
    QFormLayout *formLayout_2;
    QLabel *lbl_MinimumFontSize;
    QHBoxLayout *hboxLayout;
    QSlider *slider_minimumFontSize;
    QSpinBox *kcfg_MinimumFontSize;
    QLabel *lbl_MediumFontSize;
    QHBoxLayout *hboxLayout1;
    QSlider *slider_mediumFontSize;
    QSpinBox *kcfg_MediumFontSize;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *textLabel1;
    QFontComboBox *kcfg_StandardFont;
    QLabel *textLabel2_2;
    QFontComboBox *kcfg_FixedFont;
    QLabel *textLabel3;
    QFontComboBox *kcfg_SerifFont;
    QLabel *textLabel4;
    QFontComboBox *kcfg_SansSerifFont;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Akregator__SettingsAppearance)
    {
        if (Akregator__SettingsAppearance->objectName().isEmpty())
            Akregator__SettingsAppearance->setObjectName("Akregator__SettingsAppearance");
        Akregator__SettingsAppearance->resize(438, 538);
        verticalLayout = new QVBoxLayout(Akregator__SettingsAppearance);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox_3 = new QGroupBox(Akregator__SettingsAppearance);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout = new QGridLayout(groupBox_3);
        gridLayout->setObjectName("gridLayout");
        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setContentsMargins(0, 0, 0, 0);
        lbl_unreadArticles = new QLabel(groupBox_3);
        lbl_unreadArticles->setObjectName("lbl_unreadArticles");
        lbl_unreadArticles->setEnabled(false);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, lbl_unreadArticles);

        kcfg_ColorUnreadArticles = new KColorButton(groupBox_3);
        kcfg_ColorUnreadArticles->setObjectName("kcfg_ColorUnreadArticles");
        kcfg_ColorUnreadArticles->setEnabled(false);
        kcfg_ColorUnreadArticles->setMinimumSize(QSize(24, 24));

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_ColorUnreadArticles);

        lbl_newArticles = new QLabel(groupBox_3);
        lbl_newArticles->setObjectName("lbl_newArticles");
        lbl_newArticles->setEnabled(false);

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, lbl_newArticles);

        kcfg_ColorNewArticles = new KColorButton(groupBox_3);
        kcfg_ColorNewArticles->setObjectName("kcfg_ColorNewArticles");
        kcfg_ColorNewArticles->setEnabled(false);
        kcfg_ColorNewArticles->setMinimumSize(QSize(24, 24));

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_ColorNewArticles);


        gridLayout->addLayout(formLayout_3, 1, 0, 1, 1);

        kcfg_UseCustomColors = new QCheckBox(groupBox_3);
        kcfg_UseCustomColors->setObjectName("kcfg_UseCustomColors");

        gridLayout->addWidget(kcfg_UseCustomColors, 0, 0, 1, 1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 1, 1, 1, 1);


        verticalLayout->addWidget(groupBox_3);

        groupBox_2 = new QGroupBox(Akregator__SettingsAppearance);
        groupBox_2->setObjectName("groupBox_2");
        formLayout_2 = new QFormLayout(groupBox_2);
        formLayout_2->setObjectName("formLayout_2");
        lbl_MinimumFontSize = new QLabel(groupBox_2);
        lbl_MinimumFontSize->setObjectName("lbl_MinimumFontSize");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, lbl_MinimumFontSize);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        slider_minimumFontSize = new QSlider(groupBox_2);
        slider_minimumFontSize->setObjectName("slider_minimumFontSize");
        slider_minimumFontSize->setMinimum(2);
        slider_minimumFontSize->setMaximum(30);
        slider_minimumFontSize->setValue(8);
        slider_minimumFontSize->setOrientation(Qt::Horizontal);
        slider_minimumFontSize->setTickInterval(3);

        hboxLayout->addWidget(slider_minimumFontSize);

        kcfg_MinimumFontSize = new QSpinBox(groupBox_2);
        kcfg_MinimumFontSize->setObjectName("kcfg_MinimumFontSize");
        kcfg_MinimumFontSize->setValue(8);

        hboxLayout->addWidget(kcfg_MinimumFontSize);


        formLayout_2->setLayout(0, QFormLayout::ItemRole::FieldRole, hboxLayout);

        lbl_MediumFontSize = new QLabel(groupBox_2);
        lbl_MediumFontSize->setObjectName("lbl_MediumFontSize");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, lbl_MediumFontSize);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        slider_mediumFontSize = new QSlider(groupBox_2);
        slider_mediumFontSize->setObjectName("slider_mediumFontSize");
        slider_mediumFontSize->setMinimum(2);
        slider_mediumFontSize->setMaximum(30);
        slider_mediumFontSize->setValue(12);
        slider_mediumFontSize->setOrientation(Qt::Horizontal);
        slider_mediumFontSize->setTickInterval(3);

        hboxLayout1->addWidget(slider_mediumFontSize);

        kcfg_MediumFontSize = new QSpinBox(groupBox_2);
        kcfg_MediumFontSize->setObjectName("kcfg_MediumFontSize");
        kcfg_MediumFontSize->setValue(12);

        hboxLayout1->addWidget(kcfg_MediumFontSize);


        formLayout_2->setLayout(2, QFormLayout::ItemRole::FieldRole, hboxLayout1);


        verticalLayout->addWidget(groupBox_2);

        groupBox = new QGroupBox(Akregator__SettingsAppearance);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        textLabel1 = new QLabel(groupBox);
        textLabel1->setObjectName("textLabel1");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, textLabel1);

        kcfg_StandardFont = new QFontComboBox(groupBox);
        kcfg_StandardFont->setObjectName("kcfg_StandardFont");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_StandardFont);

        textLabel2_2 = new QLabel(groupBox);
        textLabel2_2->setObjectName("textLabel2_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, textLabel2_2);

        kcfg_FixedFont = new QFontComboBox(groupBox);
        kcfg_FixedFont->setObjectName("kcfg_FixedFont");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_FixedFont);

        textLabel3 = new QLabel(groupBox);
        textLabel3->setObjectName("textLabel3");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, textLabel3);

        kcfg_SerifFont = new QFontComboBox(groupBox);
        kcfg_SerifFont->setObjectName("kcfg_SerifFont");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, kcfg_SerifFont);

        textLabel4 = new QLabel(groupBox);
        textLabel4->setObjectName("textLabel4");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, textLabel4);

        kcfg_SansSerifFont = new QFontComboBox(groupBox);
        kcfg_SansSerifFont->setObjectName("kcfg_SansSerifFont");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, kcfg_SansSerifFont);


        verticalLayout->addWidget(groupBox);

        spacerItem1 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem1);

        verticalLayout->setStretch(3, 1);
        QWidget::setTabOrder(kcfg_UseCustomColors, kcfg_ColorUnreadArticles);
        QWidget::setTabOrder(kcfg_ColorUnreadArticles, kcfg_ColorNewArticles);
        QWidget::setTabOrder(kcfg_ColorNewArticles, slider_minimumFontSize);
        QWidget::setTabOrder(slider_minimumFontSize, kcfg_MinimumFontSize);
        QWidget::setTabOrder(kcfg_MinimumFontSize, slider_mediumFontSize);
        QWidget::setTabOrder(slider_mediumFontSize, kcfg_MediumFontSize);
        QWidget::setTabOrder(kcfg_MediumFontSize, kcfg_StandardFont);
        QWidget::setTabOrder(kcfg_StandardFont, kcfg_FixedFont);
        QWidget::setTabOrder(kcfg_FixedFont, kcfg_SerifFont);
        QWidget::setTabOrder(kcfg_SerifFont, kcfg_SansSerifFont);

        retranslateUi(Akregator__SettingsAppearance);

        QMetaObject::connectSlotsByName(Akregator__SettingsAppearance);
    } // setupUi

    void retranslateUi(QWidget *Akregator__SettingsAppearance)
    {
        groupBox_3->setTitle(tr2i18n("Article List Colors", nullptr));
        lbl_unreadArticles->setText(tr2i18n("Unread articles:", nullptr));
        lbl_newArticles->setText(tr2i18n("New articles:", nullptr));
        kcfg_UseCustomColors->setText(tr2i18n("Use custom colors", nullptr));
        groupBox_2->setTitle(tr2i18n("Font Size", nullptr));
        lbl_MinimumFontSize->setText(tr2i18n("Minimum font size:", nullptr));
        lbl_MediumFontSize->setText(tr2i18n("Medium font size:", nullptr));
        groupBox->setTitle(tr2i18n("Fonts", nullptr));
        textLabel1->setText(tr2i18n("Standard font:", nullptr));
        textLabel2_2->setText(tr2i18n("Fixed font:", nullptr));
        textLabel3->setText(tr2i18n("Serif font:", nullptr));
        textLabel4->setText(tr2i18n("Sans serif font:", nullptr));
        (void)Akregator__SettingsAppearance;
    } // retranslateUi

};

} // namespace Akregator

namespace Akregator {
namespace Ui {
    class SettingsAppearance: public Ui_SettingsAppearance {};
} // namespace Ui
} // namespace Akregator

#endif // SETTINGS_APPEARANCE_H

