/****************************************************************************
** Meta object code from reading C++ file 'framemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/frame/framemanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'framemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator12FrameManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::FrameManager::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator12FrameManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::FrameManager",
        "signalFrameAdded",
        "",
        "Akregator::Frame*",
        "signalFrameRemoved",
        "id",
        "signalRequestNewFrame",
        "int&",
        "signalSelectFrame",
        "signalCurrentFrameChanged",
        "deactivated",
        "activated",
        "signalStarted",
        "signalCanceled",
        "signalCompleted",
        "signalCaptionChanged",
        "signalTitleChanged",
        "signalIconChanged",
        "QIcon",
        "signalLoadingProgress",
        "signalStatusText",
        "slotAddFrame",
        "frame",
        "slotRemoveFrame",
        "frameId",
        "slotChangeFrame",
        "slotOpenUrlRequest",
        "Akregator::OpenUrlRequest&",
        "request",
        "useOpenInBackgroundSetting",
        "openUrl",
        "slotSetStarted",
        "slotSetCanceled",
        "reason",
        "slotSetCompleted",
        "slotSetProgress",
        "progress",
        "slotSetCaption",
        "caption",
        "slotSetTitle",
        "title",
        "slotSetStatusText",
        "statusText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalFrameAdded'
        QtMocHelpers::SignalData<void(Akregator::Frame *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalFrameRemoved'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'signalRequestNewFrame'
        QtMocHelpers::SignalData<void(int &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 5 },
        }}),
        // Signal 'signalSelectFrame'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'signalCurrentFrameChanged'
        QtMocHelpers::SignalData<void(Akregator::Frame *, Akregator::Frame *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 }, { 0x80000000 | 3, 11 },
        }}),
        // Signal 'signalStarted'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalCanceled'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'signalCompleted'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalCaptionChanged'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'signalTitleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'signalIconChanged'
        QtMocHelpers::SignalData<void(const QIcon &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'signalLoadingProgress'
        QtMocHelpers::SignalData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'signalStatusText'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotAddFrame'
        QtMocHelpers::SlotData<void(Akregator::Frame *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'slotRemoveFrame'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'slotChangeFrame'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'slotOpenUrlRequest'
        QtMocHelpers::SlotData<void(Akregator::OpenUrlRequest &, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'slotOpenUrlRequest'
        QtMocHelpers::SlotData<void(Akregator::OpenUrlRequest &)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(Akregator::OpenUrlRequest &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'slotSetStarted'
        QtMocHelpers::SlotData<void(Akregator::Frame *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'slotSetCanceled'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QString &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::QString, 33 },
        }}),
        // Slot 'slotSetCompleted'
        QtMocHelpers::SlotData<void(Akregator::Frame *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 },
        }}),
        // Slot 'slotSetProgress'
        QtMocHelpers::SlotData<void(Akregator::Frame *, int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::Int, 36 },
        }}),
        // Slot 'slotSetCaption'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QString &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'slotSetTitle'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QString &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'slotSetStatusText'
        QtMocHelpers::SlotData<void(Akregator::Frame *, const QString &)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 22 }, { QMetaType::QString, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FrameManager, qt_meta_tag_ZN9Akregator12FrameManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::FrameManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator12FrameManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator12FrameManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator12FrameManagerE_t>.metaTypes,
    nullptr
} };

void Akregator::FrameManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FrameManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalFrameAdded((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 1: _t->signalFrameRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->signalRequestNewFrame((*reinterpret_cast<std::add_pointer_t<int&>>(_a[1]))); break;
        case 3: _t->signalSelectFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->signalCurrentFrameChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[2]))); break;
        case 5: _t->signalStarted(); break;
        case 6: _t->signalCanceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->signalCompleted(); break;
        case 8: _t->signalCaptionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->signalTitleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->signalIconChanged((*reinterpret_cast<std::add_pointer_t<QIcon>>(_a[1]))); break;
        case 11: _t->signalLoadingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->signalStatusText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotAddFrame((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 14: _t->slotRemoveFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->slotChangeFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->slotOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 18: _t->openUrl((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 19: _t->slotSetStarted((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 20: _t->slotSetCanceled((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 21: _t->slotSetCompleted((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 22: _t->slotSetProgress((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->slotSetCaption((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 24: _t->slotSetTitle((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 25: _t->slotSetStatusText((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(Akregator::Frame * )>(_a, &FrameManager::signalFrameAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(int )>(_a, &FrameManager::signalFrameRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(int & )>(_a, &FrameManager::signalRequestNewFrame, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(int )>(_a, &FrameManager::signalSelectFrame, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(Akregator::Frame * , Akregator::Frame * )>(_a, &FrameManager::signalCurrentFrameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)()>(_a, &FrameManager::signalStarted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(const QString & )>(_a, &FrameManager::signalCanceled, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)()>(_a, &FrameManager::signalCompleted, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(const QString & )>(_a, &FrameManager::signalCaptionChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(const QString & )>(_a, &FrameManager::signalTitleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(const QIcon & )>(_a, &FrameManager::signalIconChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(int )>(_a, &FrameManager::signalLoadingProgress, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameManager::*)(const QString & )>(_a, &FrameManager::signalStatusText, 12))
            return;
    }
}

const QMetaObject *Akregator::FrameManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::FrameManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator12FrameManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Akregator::FrameManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void Akregator::FrameManager::signalFrameAdded(Akregator::Frame * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::FrameManager::signalFrameRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::FrameManager::signalRequestNewFrame(int & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::FrameManager::signalSelectFrame(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Akregator::FrameManager::signalCurrentFrameChanged(Akregator::Frame * _t1, Akregator::Frame * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void Akregator::FrameManager::signalStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Akregator::FrameManager::signalCanceled(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Akregator::FrameManager::signalCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Akregator::FrameManager::signalCaptionChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Akregator::FrameManager::signalTitleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Akregator::FrameManager::signalIconChanged(const QIcon & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Akregator::FrameManager::signalLoadingProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Akregator::FrameManager::signalStatusText(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
