/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.akregator.part.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef PARTADAPTOR_H
#define PARTADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "akregator_part.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.akregator.part
 */
class PartAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.akregator.part")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.akregator.part\">\n"
"    <method name=\"openStandardFeedList\"/>\n"
"    <method name=\"fetchFeedUrl\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"url\"/>\n"
"    </method>\n"
"    <method name=\"fetchAllFeeds\"/>\n"
"    <method name=\"saveSettings\"/>\n"
"    <method name=\"addFeedsToGroup\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"lst\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"feedname\"/>\n"
"    </method>\n"
"    <method name=\"exportFile\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"url\"/>\n"
"    </method>\n"
"    <method name=\"addFeed\"/>\n"
"    <method name=\"handleCommandLine\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"args\"/>\n"
"      <arg direction=\"out\" type=\"b\" name=\"result\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    PartAdaptor(Akregator::Part *parent);
    ~PartAdaptor() override;

    inline Akregator::Part *parent() const
    { return static_cast<Akregator::Part *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void addFeed();
    void addFeedsToGroup(const QStringList &lst, const QString &feedname);
    void exportFile(const QString &url);
    void fetchAllFeeds();
    void fetchFeedUrl(const QString &url);
    bool handleCommandLine(const QStringList &args);
    void openStandardFeedList();
    void saveSettings();
Q_SIGNALS: // SIGNALS
};

#endif
