/****************************************************************************
** Meta object code from reading C++ file 'frame.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/frame/frame.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'frame.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator5FrameE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::Frame::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator5FrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::Frame",
        "signalCaptionChanged",
        "",
        "Akregator::Frame*",
        "signalTitleChanged",
        "signalStarted",
        "signalCanceled",
        "signalCompleted",
        "signalLoadingProgress",
        "signalStatusText",
        "signalOpenUrlRequest",
        "Akregator::OpenUrlRequest&",
        "request",
        "showStatusBarMessage",
        "msg",
        "slotReload",
        "slotStop",
        "slotSetStarted",
        "slotSetCanceled",
        "slotSetCompleted",
        "slotSetState",
        "Akregator::Frame::State",
        "slotSetProgress",
        "slotSetCaption",
        "slotSetTitle",
        "slotSetStatusText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalCaptionChanged'
        QtMocHelpers::SignalData<void(Akregator::Frame *, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'signalTitleChanged'
        QtMocHelpers::SignalData<void(Akregator::Frame *, const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'signalStarted'
        QtMocHelpers::SignalData<void(Akregator::Frame *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalCanceled'
        QtMocHelpers::SignalData<void(Akregator::Frame *, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'signalCompleted'
        QtMocHelpers::SignalData<void(Akregator::Frame *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'signalLoadingProgress'
        QtMocHelpers::SignalData<void(Akregator::Frame *, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'signalStatusText'
        QtMocHelpers::SignalData<void(Akregator::Frame *, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'signalOpenUrlRequest'
        QtMocHelpers::SignalData<void(Akregator::OpenUrlRequest &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'showStatusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'slotReload'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStop'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetStarted'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetCanceled'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSetCompleted'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetState'
        QtMocHelpers::SlotData<void(Akregator::Frame::State)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 2 },
        }}),
        // Slot 'slotSetProgress'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSetCaption'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSetTitle'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotSetStatusText'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Frame, qt_meta_tag_ZN9Akregator5FrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::Frame::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator5FrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator5FrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator5FrameE_t>.metaTypes,
    nullptr
} };

void Akregator::Frame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Frame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalCaptionChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->signalTitleChanged((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->signalStarted((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 3: _t->signalCanceled((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->signalCompleted((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1]))); break;
        case 5: _t->signalLoadingProgress((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->signalStatusText((*reinterpret_cast<std::add_pointer_t<Akregator::Frame*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->signalOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 8: _t->showStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->slotReload(); break;
        case 10: _t->slotStop(); break;
        case 11: _t->slotSetStarted(); break;
        case 12: _t->slotSetCanceled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotSetCompleted(); break;
        case 14: _t->slotSetState((*reinterpret_cast<std::add_pointer_t<Akregator::Frame::State>>(_a[1]))); break;
        case 15: _t->slotSetProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotSetCaption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->slotSetTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->slotSetStatusText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::Frame* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * , const QString & )>(_a, &Frame::signalCaptionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * , const QString & )>(_a, &Frame::signalTitleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * )>(_a, &Frame::signalStarted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * , const QString & )>(_a, &Frame::signalCanceled, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * )>(_a, &Frame::signalCompleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * , int )>(_a, &Frame::signalLoadingProgress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::Frame * , const QString & )>(_a, &Frame::signalStatusText, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(Akregator::OpenUrlRequest & )>(_a, &Frame::signalOpenUrlRequest, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Frame::*)(const QString & )>(_a, &Frame::showStatusBarMessage, 8))
            return;
    }
}

const QMetaObject *Akregator::Frame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::Frame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator5FrameE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Akregator::Frame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void Akregator::Frame::signalCaptionChanged(Akregator::Frame * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Akregator::Frame::signalTitleChanged(Akregator::Frame * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Akregator::Frame::signalStarted(Akregator::Frame * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::Frame::signalCanceled(Akregator::Frame * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Akregator::Frame::signalCompleted(Akregator::Frame * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Akregator::Frame::signalLoadingProgress(Akregator::Frame * _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void Akregator::Frame::signalStatusText(Akregator::Frame * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void Akregator::Frame::signalOpenUrlRequest(Akregator::OpenUrlRequest & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Akregator::Frame::showStatusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
