/********************************************************************************
** Form generated from reading UI file 'settings_security.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_SECURITY_H
#define UI_SETTINGS_SECURITY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SettingsSecurity
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_LoadExternalReferences;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SettingsSecurity)
    {
        if (SettingsSecurity->objectName().isEmpty())
            SettingsSecurity->setObjectName("SettingsSecurity");
        SettingsSecurity->resize(400, 300);
        verticalLayout = new QVBoxLayout(SettingsSecurity);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_LoadExternalReferences = new QCheckBox(SettingsSecurity);
        kcfg_LoadExternalReferences->setObjectName("kcfg_LoadExternalReferences");

        verticalLayout->addWidget(kcfg_LoadExternalReferences);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(SettingsSecurity);

        QMetaObject::connectSlotsByName(SettingsSecurity);
    } // setupUi

    void retranslateUi(QWidget *SettingsSecurity)
    {
        kcfg_LoadExternalReferences->setText(tr2i18n("Allow article preview to load external references", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_LoadExternalReferences->setToolTip(tr2i18n("Allow the article preview to load external references from the Internet. This may allow websites or third parties to observe the articles that you have viewed.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)SettingsSecurity;
    } // retranslateUi

};

namespace Ui {
    class SettingsSecurity: public Ui_SettingsSecurity {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SETTINGS_SECURITY_H

