/****************************************************************************
** Meta object code from reading C++ file 'mainwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/mainwidget.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator10MainWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::MainWidget::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator10MainWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::MainWidget",
        "signalUnreadCountChanged",
        "",
        "signalArticlesSelected",
        "QList<Akregator::Article>",
        "slotOpenSelectedArticlesInBrowser",
        "slotOpenSelectedArticles",
        "slotOpenSelectedArticlesInBackground",
        "slotOnShutdown",
        "slotNodeSelected",
        "Akregator::TreeNode*",
        "node",
        "slotArticleSelected",
        "Akregator::Article",
        "ensureArticleTabVisible",
        "slotSetTotalUnread",
        "slotCopyLinkAddress",
        "slotRequestNewFrame",
        "int&",
        "frameId",
        "slotFeedAdd",
        "slotFeedAddGroup",
        "slotFeedRemove",
        "slotFeedModify",
        "slotFetchCurrentFeed",
        "slotFetchAllFeeds",
        "slotMarkAllRead",
        "slotMarkAllFeedsRead",
        "slotOpenHomepage",
        "slotReloadAllTabs",
        "slotArticleToggleKeepFlag",
        "enabled",
        "slotArticleDelete",
        "slotSetSelectedArticleRead",
        "slotSetSelectedArticleUnread",
        "slotSetSelectedArticleNew",
        "slotSetCurrentArticleReadDelayed",
        "slotNormalView",
        "slotWidescreenView",
        "slotCombinedView",
        "slotToggleShowQuickFilter",
        "slotPrevUnreadArticle",
        "slotNextUnreadArticle",
        "slotMoveCurrentNodeUp",
        "slotMoveCurrentNodeDown",
        "slotMoveCurrentNodeLeft",
        "slotMoveCurrentNodeRight",
        "slotSendLink",
        "slotSendFile",
        "slotNetworkStatusChanged",
        "status",
        "slotFocusQuickSearch",
        "slotWhatsNew",
        "slotMouseButtonPressed",
        "button",
        "QUrl",
        "slotOpenArticleInBrowser",
        "article",
        "slotDoIntervalFetches",
        "slotDeleteExpiredArticles",
        "slotFetchingStarted",
        "slotFetchingStopped",
        "slotFramesChanged",
        "slotShowStatusBarMessage",
        "msg",
        "slotCurrentFrameChanged",
        "slotArticleAction",
        "Akregator::ArticleViewerWebEngine::ArticleAction",
        "type",
        "articleId",
        "feed",
        "slotSettingsChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalUnreadCountChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'signalArticlesSelected'
        QtMocHelpers::SignalData<void(const QList<Akregator::Article> &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotOpenSelectedArticlesInBrowser'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenSelectedArticles'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenSelectedArticlesInBackground'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnShutdown'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNodeSelected'
        QtMocHelpers::SlotData<void(Akregator::TreeNode *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotArticleSelected'
        QtMocHelpers::SlotData<void(const Akregator::Article &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'ensureArticleTabVisible'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetTotalUnread'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCopyLinkAddress'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRequestNewFrame'
        QtMocHelpers::SlotData<void(int &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'slotFeedAdd'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFeedAddGroup'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFeedRemove'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFeedModify'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFetchCurrentFeed'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFetchAllFeeds'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMarkAllRead'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMarkAllFeedsRead'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOpenHomepage'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReloadAllTabs'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotArticleToggleKeepFlag'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'slotArticleDelete'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetSelectedArticleRead'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetSelectedArticleUnread'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetSelectedArticleNew'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetCurrentArticleReadDelayed'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNormalView'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWidescreenView'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCombinedView'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleShowQuickFilter'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPrevUnreadArticle'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextUnreadArticle'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMoveCurrentNodeUp'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMoveCurrentNodeDown'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMoveCurrentNodeLeft'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMoveCurrentNodeRight'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSendLink'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSendFile'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNetworkStatusChanged'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 50 },
        }}),
        // Slot 'slotFocusQuickSearch'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWhatsNew'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMouseButtonPressed'
        QtMocHelpers::SlotData<void(int, const QUrl &)>(53, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 54 }, { 0x80000000 | 55, 2 },
        }}),
        // Slot 'slotOpenArticleInBrowser'
        QtMocHelpers::SlotData<void(const Akregator::Article &)>(56, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 57 },
        }}),
        // Slot 'slotDoIntervalFetches'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDeleteExpiredArticles'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFetchingStarted'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFetchingStopped'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFramesChanged'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowStatusBarMessage'
        QtMocHelpers::SlotData<void(const QString &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Slot 'slotCurrentFrameChanged'
        QtMocHelpers::SlotData<void(int)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'slotArticleAction'
        QtMocHelpers::SlotData<void(Akregator::ArticleViewerWebEngine::ArticleAction, const QString &, const QString &)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 67, 68 }, { QMetaType::QString, 69 }, { QMetaType::QString, 70 },
        }}),
        // Slot 'slotSettingsChanged'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWidget, qt_meta_tag_ZN9Akregator10MainWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::MainWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator10MainWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator10MainWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator10MainWidgetE_t>.metaTypes,
    nullptr
} };

void Akregator::MainWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalUnreadCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->signalArticlesSelected((*reinterpret_cast<std::add_pointer_t<QList<Akregator::Article>>>(_a[1]))); break;
        case 2: _t->slotOpenSelectedArticlesInBrowser(); break;
        case 3: _t->slotOpenSelectedArticles(); break;
        case 4: _t->slotOpenSelectedArticlesInBackground(); break;
        case 5: _t->slotOnShutdown(); break;
        case 6: _t->slotNodeSelected((*reinterpret_cast<std::add_pointer_t<Akregator::TreeNode*>>(_a[1]))); break;
        case 7: _t->slotArticleSelected((*reinterpret_cast<std::add_pointer_t<Akregator::Article>>(_a[1]))); break;
        case 8: _t->ensureArticleTabVisible(); break;
        case 9: _t->slotSetTotalUnread(); break;
        case 10: _t->slotCopyLinkAddress(); break;
        case 11: _t->slotRequestNewFrame((*reinterpret_cast<std::add_pointer_t<int&>>(_a[1]))); break;
        case 12: _t->slotFeedAdd(); break;
        case 13: _t->slotFeedAddGroup(); break;
        case 14: _t->slotFeedRemove(); break;
        case 15: _t->slotFeedModify(); break;
        case 16: _t->slotFetchCurrentFeed(); break;
        case 17: _t->slotFetchAllFeeds(); break;
        case 18: _t->slotMarkAllRead(); break;
        case 19: _t->slotMarkAllFeedsRead(); break;
        case 20: _t->slotOpenHomepage(); break;
        case 21: _t->slotReloadAllTabs(); break;
        case 22: _t->slotArticleToggleKeepFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->slotArticleDelete(); break;
        case 24: _t->slotSetSelectedArticleRead(); break;
        case 25: _t->slotSetSelectedArticleUnread(); break;
        case 26: _t->slotSetSelectedArticleNew(); break;
        case 27: _t->slotSetCurrentArticleReadDelayed(); break;
        case 28: _t->slotNormalView(); break;
        case 29: _t->slotWidescreenView(); break;
        case 30: _t->slotCombinedView(); break;
        case 31: _t->slotToggleShowQuickFilter(); break;
        case 32: _t->slotPrevUnreadArticle(); break;
        case 33: _t->slotNextUnreadArticle(); break;
        case 34: _t->slotMoveCurrentNodeUp(); break;
        case 35: _t->slotMoveCurrentNodeDown(); break;
        case 36: _t->slotMoveCurrentNodeLeft(); break;
        case 37: _t->slotMoveCurrentNodeRight(); break;
        case 38: _t->slotSendLink(); break;
        case 39: _t->slotSendFile(); break;
        case 40: _t->slotNetworkStatusChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->slotFocusQuickSearch(); break;
        case 42: _t->slotWhatsNew(); break;
        case 43: _t->slotMouseButtonPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2]))); break;
        case 44: _t->slotOpenArticleInBrowser((*reinterpret_cast<std::add_pointer_t<Akregator::Article>>(_a[1]))); break;
        case 45: _t->slotDoIntervalFetches(); break;
        case 46: _t->slotDeleteExpiredArticles(); break;
        case 47: _t->slotFetchingStarted(); break;
        case 48: _t->slotFetchingStopped(); break;
        case 49: _t->slotFramesChanged(); break;
        case 50: _t->slotShowStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: _t->slotCurrentFrameChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 52: _t->slotArticleAction((*reinterpret_cast<std::add_pointer_t<Akregator::ArticleViewerWebEngine::ArticleAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 53: _t->slotSettingsChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Akregator::TreeNode* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWidget::*)(int )>(_a, &MainWidget::signalUnreadCountChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWidget::*)(const QList<Akregator::Article> & )>(_a, &MainWidget::signalArticlesSelected, 1))
            return;
    }
}

const QMetaObject *Akregator::MainWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::MainWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator10MainWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Akregator::MainWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 54)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 54;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 54)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 54;
    }
    return _id;
}

// SIGNAL 0
void Akregator::MainWidget::signalUnreadCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::MainWidget::signalArticlesSelected(const QList<Akregator::Article> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
