/****************************************************************************
** Meta object code from reading C++ file 'articleviewerwebengine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/articleviewer-ng/webengine/articleviewerwebengine.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'articleviewerwebengine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::ArticleViewerWebEngine::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::ArticleViewerWebEngine",
        "signalOpenUrlRequest",
        "",
        "Akregator::OpenUrlRequest&",
        "showStatusBarMessage",
        "link",
        "showContextMenu",
        "QPoint",
        "pos",
        "articleAction",
        "Akregator::ArticleViewerWebEngine::ArticleAction",
        "type",
        "articleId",
        "feed",
        "findTextInHtml",
        "textToSpeech",
        "webPageMutedOrAudibleChanged",
        "isAudioMuted",
        "wasRecentlyAudible",
        "trackingFound",
        "WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList",
        "slotCopy",
        "slotMute",
        "mute",
        "slotServiceUrlSelected",
        "PimCommon::ShareServiceUrlManager::ServiceType",
        "slotLinkHovered",
        "slotLoadStarted",
        "slotLoadFinished",
        "slotLinkClicked",
        "QUrl",
        "url",
        "slotShowContextMenu",
        "slotWebHitFinished",
        "WebEngineViewer::WebHitTestResult",
        "result",
        "slotActivatePlugin",
        "MessageViewer::ViewerPluginInterface*",
        "interface",
        "slotWebPageMutedOrAudibleChanged",
        "slotCheckedUrlFinished",
        "WebEngineViewer::CheckPhishingUrlUtil::UrlStatus",
        "status"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalOpenUrlRequest'
        QtMocHelpers::SignalData<void(Akregator::OpenUrlRequest &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'showStatusBarMessage'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'showContextMenu'
        QtMocHelpers::SignalData<void(const QPoint &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'articleAction'
        QtMocHelpers::SignalData<void(Akregator::ArticleViewerWebEngine::ArticleAction, const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'findTextInHtml'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'textToSpeech'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'webPageMutedOrAudibleChanged'
        QtMocHelpers::SignalData<void(bool, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'trackingFound'
        QtMocHelpers::SignalData<void(const WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Slot 'slotCopy'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMute'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'slotServiceUrlSelected'
        QtMocHelpers::SlotData<void(PimCommon::ShareServiceUrlManager::ServiceType)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 11 },
        }}),
        // Slot 'slotLinkHovered'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slotLoadStarted'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadFinished'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotShowContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotWebHitFinished'
        QtMocHelpers::SlotData<void(const WebEngineViewer::WebHitTestResult &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'slotActivatePlugin'
        QtMocHelpers::SlotData<void(MessageViewer::ViewerPluginInterface *)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'slotWebPageMutedOrAudibleChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCheckedUrlFinished'
        QtMocHelpers::SlotData<void(const QUrl &, WebEngineViewer::CheckPhishingUrlUtil::UrlStatus)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 41, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ArticleViewerWebEngine, qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::ArticleViewerWebEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<WebEngineViewer::WebEngineView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>.metaTypes,
    nullptr
} };

void Akregator::ArticleViewerWebEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArticleViewerWebEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalOpenUrlRequest((*reinterpret_cast<std::add_pointer_t<Akregator::OpenUrlRequest&>>(_a[1]))); break;
        case 1: _t->showStatusBarMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->showContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->articleAction((*reinterpret_cast<std::add_pointer_t<Akregator::ArticleViewerWebEngine::ArticleAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->findTextInHtml(); break;
        case 5: _t->textToSpeech(); break;
        case 6: _t->webPageMutedOrAudibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->trackingFound((*reinterpret_cast<std::add_pointer_t<WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList>>(_a[1]))); break;
        case 8: _t->slotCopy(); break;
        case 9: _t->slotMute((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotServiceUrlSelected((*reinterpret_cast<std::add_pointer_t<PimCommon::ShareServiceUrlManager::ServiceType>>(_a[1]))); break;
        case 11: _t->slotLinkHovered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->slotLoadStarted(); break;
        case 13: _t->slotLoadFinished(); break;
        case 14: _t->slotLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 15: _t->slotShowContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 16: _t->slotWebHitFinished((*reinterpret_cast<std::add_pointer_t<WebEngineViewer::WebHitTestResult>>(_a[1]))); break;
        case 17: _t->slotActivatePlugin((*reinterpret_cast<std::add_pointer_t<MessageViewer::ViewerPluginInterface*>>(_a[1]))); break;
        case 18: _t->slotWebPageMutedOrAudibleChanged(); break;
        case 19: _t->slotCheckedUrlFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WebEngineViewer::CheckPhishingUrlUtil::UrlStatus>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PimCommon::ShareServiceUrlManager::ServiceType >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageViewer::ViewerPluginInterface* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WebEngineViewer::CheckPhishingUrlUtil::UrlStatus >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(Akregator::OpenUrlRequest & )>(_a, &ArticleViewerWebEngine::signalOpenUrlRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(const QString & )>(_a, &ArticleViewerWebEngine::showStatusBarMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(const QPoint & )>(_a, &ArticleViewerWebEngine::showContextMenu, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(Akregator::ArticleViewerWebEngine::ArticleAction , const QString & , const QString & )>(_a, &ArticleViewerWebEngine::articleAction, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)()>(_a, &ArticleViewerWebEngine::findTextInHtml, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)()>(_a, &ArticleViewerWebEngine::textToSpeech, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(bool , bool )>(_a, &ArticleViewerWebEngine::webPageMutedOrAudibleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArticleViewerWebEngine::*)(const WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList & )>(_a, &ArticleViewerWebEngine::trackingFound, 7))
            return;
    }
}

const QMetaObject *Akregator::ArticleViewerWebEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::ArticleViewerWebEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator22ArticleViewerWebEngineE_t>.strings))
        return static_cast<void*>(this);
    return WebEngineViewer::WebEngineView::qt_metacast(_clname);
}

int Akregator::ArticleViewerWebEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WebEngineViewer::WebEngineView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void Akregator::ArticleViewerWebEngine::signalOpenUrlRequest(Akregator::OpenUrlRequest & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Akregator::ArticleViewerWebEngine::showStatusBarMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Akregator::ArticleViewerWebEngine::showContextMenu(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Akregator::ArticleViewerWebEngine::articleAction(Akregator::ArticleViewerWebEngine::ArticleAction _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void Akregator::ArticleViewerWebEngine::findTextInHtml()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Akregator::ArticleViewerWebEngine::textToSpeech()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Akregator::ArticleViewerWebEngine::webPageMutedOrAudibleChanged(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void Akregator::ArticleViewerWebEngine::trackingFound(const WebEngineViewer::BlockTrackingUrlInterceptor::TrackerBlackList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
