/****************************************************************************
** Meta object code from reading C++ file 'articlegrantleeobject.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../akregator-26.07.90/src/formatter/articlegrantleeobject.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'articlegrantleeobject.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto Akregator::ArticleGrantleeObject::qt_create_metaobjectdata<qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Akregator::ArticleGrantleeObject",
        "strippedTitle",
        "author",
        "content",
        "articleLinkUrl",
        "articlePubDate",
        "enclosure",
        "articleCompleteStoryLink",
        "imageFeed",
        "articleStatus",
        "Akregator::ArticleGrantleeObject::ArticleStatus",
        "important",
        "actionToken",
        "ArticleStatus",
        "Unread",
        "Read",
        "New"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'strippedTitle'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'author'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'content'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'articleLinkUrl'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'articlePubDate'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'enclosure'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'articleCompleteStoryLink'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'imageFeed'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'articleStatus'
        QtMocHelpers::PropertyData<Akregator::ArticleGrantleeObject::ArticleStatus>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'important'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'actionToken'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ArticleStatus'
        QtMocHelpers::EnumData<enum ArticleStatus>(13, 13, QMC::EnumFlags{}).add({
            {   14, ArticleStatus::Unread },
            {   15, ArticleStatus::Read },
            {   16, ArticleStatus::New },
        }),
    };
    return QtMocHelpers::metaObjectData<ArticleGrantleeObject, qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Akregator::ArticleGrantleeObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>.metaTypes,
    nullptr
} };

void Akregator::ArticleGrantleeObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArticleGrantleeObject *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->strippedTitle(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->author(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->content(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->articleLinkUrl(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->articlePubDate(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->enclosure(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->articleCompleteStoryLink(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->imageFeed(); break;
        case 8: *reinterpret_cast<Akregator::ArticleGrantleeObject::ArticleStatus*>(_v) = _t->articleStatus(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->important(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->actionToken(); break;
        default: break;
        }
    }
}

const QMetaObject *Akregator::ArticleGrantleeObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Akregator::ArticleGrantleeObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9Akregator21ArticleGrantleeObjectE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Akregator::ArticleGrantleeObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
